#include "QtInterface.h"
using namespace std;


void cubicalBox::set12Lines()
{

  point p1, p2, pt;
  float r = SQRT2*maxDT;

  p1 = l1.getP1();
  p2 = l1.getP2();
  pt = p2 - p1;
  
  SbVec3f p12(pt.getX(), pt.getY(), pt.getZ());
  SbVec3f p1x(pt.getX()+1, pt.getY()+2, pt.getZ()+3);
  SbVec3f u, v;

  u = p12.cross(p1x);
  v = p12.cross(u);

  float umag = u.normalize();
  float vmag = v.normalize();

  vector<point> p1Plane, p2Plane;

  for(float theta = 0; theta<2*PI; theta = theta + PI/2)
    {
      float x = p1.getX() + r*cos(theta)*u[0] + r*sin(theta)*v[0];
      float y = p1.getY() + r*cos(theta)*u[1] + r*sin(theta)*v[1];
      float z = p1.getZ() + r*cos(theta)*u[2] + r*sin(theta)*v[2];

      p1Plane.push_back(point(x, y, z));
      p2Plane.push_back(point(x, y, z) + pt);
     
    }

  /*-------------------------------------------------*/
  //DEBUGGING
  /*-------------------------------------------------*/
  
  for(int i = 0; i < 4; i++)
    {
      vline.push_back(line(p1Plane[i], p2Plane[i]));
      vline.push_back(line(p1Plane[i], p1Plane[(i+1)%4]));
      vline.push_back(line(p2Plane[i], p2Plane[(i+1)%4]));
    }

}


bool cubicalBox::isIntersectingZPlane(float zPlane)
{
  for(int i = 0; i < 12; i++)
    {
      if(vline[i].intersectZPlane(zPlane))
	return true;
    }
  return false;
}


vector<point> cubicalBox::getIntersectingVerticesWithZPlane( float zPlane)
{
  vector<point> vIntersect;

  for(int i = 0; i < 12; i++)
    {
      if(vline[i].intersectZPlane(zPlane))
	{	 
	  vIntersect.push_back(vline[i].intersectPoint(zPlane));
	}
    }
  return vIntersect;
}


vector<point> sortPoints(vector<point> vpoint)
{
  vector<point> sortedpoints;
  int min;
  vector<thetaIndex> vtid;
  point pmin;
  pmin.setPoint(vpoint[0]);

  for(int i=1; i<vpoint.size(); i++)
    {
      if(vpoint[i].getY() < pmin.getY())
	{
	pmin.setPoint(vpoint[i]);
	min = i;
	}
      if(vpoint[i].getY() == pmin.getY())
	if(vpoint[i].getX() > pmin.getX())
	  {
	  pmin.setPoint(vpoint[i]);
	  min = i;
	  }

    }
  
  sortedpoints.push_back(pmin);

  for(int i=0; i<vpoint.size(); i++)
    {
      SbVec3f Vd((vpoint[i].getX()-pmin.getX()), (vpoint[i].getY()-pmin.getY()), (vpoint[i].getZ()-pmin.getZ()));
      if(Vd.length() != 0.0)
	{
	  float theta = (Vd[0]>0) ? asin(Vd[1]/Vd.length()) : (PI-asin(Vd[1]/Vd.length())); 
	  vtid.push_back(thetaIndex(theta, i));
	}
    }

  sort(vtid.begin(), vtid.end());

  for(int i=0; i<vtid.size(); i++)
    sortedpoints.push_back(vpoint[vtid[i].getIndex()]);
  
  return sortedpoints;
}


vector<int> sortVertices(vector<Vertex> vlist)
{
  vector<int> sortedpoints;
  int min;
  vector<thetaIndex> vtid;
  point pmin;
  pmin.setPoint(vlist[0].getPosCoords());

  for(int i=1; i<vlist.size(); i++)
    {
      point p;
      p = vlist[i].getPosCoords();
      if(p.getY() < pmin.getY())
	{
	pmin.setPoint(p);
	min = i;
	}
      if(p.getY() == pmin.getY())
	if(p.getX() > pmin.getX())
	  {
	  pmin.setPoint(p);
	  min = i;
	  }

    }
  
  sortedpoints.push_back(min);

  for(int i=0; i<vlist.size(); i++)
    {
      point pp;
      pp = vlist[i].getPosCoords();
      SbVec3f Vd((pp.getX()-pmin.getX()), (pp.getY()-pmin.getY()), (pp.getZ()-pmin.getZ()));
      if(Vd.length() != 0.0)
	{
	  float theta = (Vd[0]>0) ? asin(Vd[1]/Vd.length()) : (PI-asin(Vd[1]/Vd.length())); 
	  vtid.push_back(thetaIndex(theta, i));
	}
    }

  sort(vtid.begin(), vtid.end());

  for(int i=0; i<vtid.size(); i++)
    sortedpoints.push_back(vtid[i].getIndex());
  
  return sortedpoints;
}
