//#include "skeletonFile.h"
#include "QtInterface.h"

using namespace std;

unsigned char* volume;
int tex_width, tex_height, tex_depth;

unsigned char* loadVolume(char* filename, int xsize, int ysize, int zsize)
{
  	
	ifstream infile(filename);	
	tex_width = xsize; tex_height = ysize; tex_depth = zsize;

	//	volume = (unsigned char*)malloc(4*xsize*ysize*zsize*sizeof(unsigned char));
	volume = new unsigned char[xsize*ysize*zsize];
	infile.read(volume, xsize*ysize*zsize*sizeof(unsigned char));

	if(!infile) {
	cerr << "\n Can't Open" << filename << endl;
	exit(-1);
	}
	
	//   return volume;

	return volume;
 
}

  
void Reconstruct::Init()
{
  xMin = yMin = zMin = 0;
}

void Reconstruct::Execute(int argc, char* argv[])
{
  xMin = yMin = zMin = 0;
  xLookupSize = atoi(argv[4]);
  yLookupSize = atoi(argv[5]);
  zLookupSize = atoi(argv[6]);
  
  xMax = xSize = xLookupSize;
  yMax = ySize = yLookupSize;
  zMax = zSize = zLookupSize;

  //Fill in the values from the command line arguments
  
  xMin = atoi(argv[7]) - PADDING;
  xMax = atoi(argv[8]) + PADDING;
  yMin = atoi(argv[9]) - PADDING;
  yMax = atoi(argv[10]) + PADDING;
  zMin = atoi(argv[11]) - PADDING;
  zMax = atoi(argv[12]) + PADDING;

  //Reconstructed Volume Size
  xSize = xMax - xMin + 1;
  ySize = yMax - yMin + 1;
  zSize = zMax - zMin + 1;

  cout<<"Command line args read: "<<endl;
  cout<<"\t Original Volume: "<<argv[2]<<" Size: "<<xLookupSize<<"x"<<yLookupSize<<"x"<<zLookupSize<<endl;
  cout<<endl<<"\t Tskel File: "<<argv[1]<<endl;
  cout<<endl<<"\t Volume Bounds: ["<<xMin<<" "<<yMin<<" "<<zMin<<"] - ["<<xMax<<" "<<yMax<<" "<<zMax<<"]"<<endl;
  cout<<"\t Reconstructed volume size: "<<xSize<<"x"<<ySize<<"x"<<zSize<<endl;


  if((xSize<=0) || (ySize<=0) || (zSize<=0) || (xLookupSize<=0) || (yLookupSize<=0) || (zLookupSize<=0))
    {
      cerr<<"Error in parameters....check usage"<<endl;
      //call back to show usage and exit
      Execute(1, argv);
      exit(1);
    }

  tskelFile = fopen(argv[1], "r");
  inFile = fopen(argv[2], "r");
  outFile = fopen(argv[3], "w");

  if(!tskelFile || !inFile || !outFile)
    {
      cerr<<"File I/O error ....quitting"<<endl;
      exit(1);
    }
 
  volumeSize = xSize*ySize*zSize;
  lookupVolumeSize = xLookupSize*yLookupSize*zLookupSize;
  theBuffer = new unsigned char[volumeSize];
  theProximityBuffer = new unsigned char[volumeSize];
  theLookupBuffer = new unsigned char[lookupVolumeSize];
  cout<<"xlookupSize "<<xLookupSize<<endl;
  cout<<"yLookupSize "<<yLookupSize<<endl;
  cout<<"zLookupSize "<<zLookupSize<<endl;
  cout<<"lookupVolumeSize "<<lookupVolumeSize<<endl;
  if((!theBuffer) || (!theProximityBuffer) || (!theLookupBuffer))
    {
      cerr<<"Failed to allocate memory....exiting"<<endl;
      exit(1);
    }
  
  for (long longCounter=0; longCounter<volumeSize; longCounter++)
    {
      theBuffer[longCounter] = 0;
      theProximityBuffer[longCounter] = 255;
    }

  
#ifdef SAMPLED
  int readBytes = fread(theLookupBuffer, sizeof(unsigned char), lookupVolumeSize, inFile);
  
  if(readBytes != lookupVolumeSize)
    {
      cerr<<"Expected "<<lookupVolumeSize<<"bytes, read only "<<readBytes<<"bytes"<<endl;
      exit(1);
    }
  cout<<"Original volume read properly..."<<endl;
#endif

  //Initializing GL
  // initGL(theLookupBuffer, xLookupSize, yLookupSize, zLookupSize);

  cout<<"Calling reconstruction function..."<<endl;
  /* skeletonFile fskel(tskelFile); */
 
/*   fskel.renderCuboids(); */
/*   fskel.defineJointGeometry(); */
/*   fskel.renderJointGeometry(); */

  /**********Clear sphere and re-render**************/  
 /*  glDisable(GL_TEXTURE_3D); */
/*   glBlendFunc(GL_ONE, GL_ZERO); */
/*   fskel.renderJointSpheres(false, 0.9999); */
/*   glEnable(GL_TEXTURE_3D); */
/*   glColor4f(0.2, 0.2, 0.3, 0.0065); */
/*   glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA); */
/*   fskel.renderJointSpheres(true, 1.0); */
  /*****************************************************/
  
  //TransformedRecon();

  //cout<<"Done reconstruction ....dumping volume to disk"<<endl;
  //fwrite(theBuffer, sizeof(unsigned char), volumeSize, outFile);

  fclose(inFile);
  fclose(tskelFile);
  fclose(outFile);
  
  cout<<"Files have been closed"<<endl;
  //cout<<"Once again: Reconstructed Volume Size: "<<xSize<<"x"<<ySize<<"x"<<zSize<<endl;
  // exit(0);
  
}


void Reconstruct::TransformedRecon()
{
  SoDB::init();
  char str[256];
  float xx, yy, zz, xx1, yy1, zz1, xx2, yy2, zz2, dt, maxDT, dist;
  int x, y, z;
  int count = 0;
  line l2;

  fscanf(tskelFile, "%[^\n]", str);
  while(!feof(tskelFile))
    {
      count++;
      if(count%50 == 0)
	{
	  // cout<<"Done "<<count<<"points DT Now: "<<dt<<endl;
	}

      if(strncmp(str, "Root", 4) == 0)
	{
	  maxDT = 0;
	  
	  //Reading in root node information
	  
	  //cout<<":: "<<str<<endl;
	  
	  if(sscanf(str, "Root %f%f%f : %f%f%f", &xx, &yy, &zz, &xx1, &yy1, &zz1) < 6)
	    {
	      cerr<<"Error in tskel file"<<endl;
	      cerr<<":- "<<str<<endl;
	      exit(1);
	    }
     

	  //Scan in the 4x4 transformation matrix
	  fscanf(tskelFile, "%f%f%f%f", &mat[0][0], &mat[0][1], &mat[0][2], &mat[0][3]);
	  fscanf(tskelFile, "%f%f%f%f", &mat[1][0], &mat[1][1], &mat[1][2], &mat[1][3]);
	  fscanf(tskelFile, "%f%f%f%f", &mat[2][0], &mat[2][1], &mat[2][2], &mat[2][3]);
	  fscanf(tskelFile, "%f%f%f%f", &mat[3][0], &mat[3][1], &mat[3][2], &mat[3][3]);

	  inv = mat.inverse();
	  l2.setP1P2(point(xx, yy, zz), point(xx1, yy1, zz1));
	  
	}

      else
	{
	  //reading in reconstruction sphere
	  xx2 = yy2 = zz2 = dt = 0;
	  if(sscanf(str, "%f%f%f%f", &xx2, &yy2, &zz2, &dt) != 4)
	    {
	      cerr<<"Error in tskel file"<<endl;
	      cerr<<":= "<<str<<endl;
	      cerr<<"values read: "<<xx2<<" "<<yy2<<" "<<zz2<<" "<<dt<<endl;
	      exit(1);
	    }

	  float dist;
	  dist = l2.lineTo3dpointDistance(point(xx2, yy2, zz2)) + dt/3 ;
	  if(dist >= maxDT)
	    {
	    maxDT = dist;
	    }
	
	 
	  SbVec3f v(xx2, yy2, zz2);
	  SbVec3f v1;
	  mat.multVecMatrix(v, v1);
	  
	  x = (int) v1[0];
	  y = (int) v1[1];
	  z = (int) v1[2];

	  //Call the reconstruction routine of choice
	  //GrowSphere(x, y, z, (int)dt);
	}
      
       
      //Proceed to taking in next line
      fscanf(tskelFile, "\n");
      fscanf(tskelFile, "%[^\n]", str);
      
      if(((strncmp(str, "Root", 4) == 0)  || feof(tskelFile)) && count != 1)
	{

	  /*-------------------Transformation to Bones-------------*/
	
	  SbVec3f bonePoint1(xx, yy, zz);
	  SbVec3f bonePoint2(xx1, yy1, zz1);
	  SbVec3f bonePointTr1, bonePointTr2;
	  mat.multVecMatrix(bonePoint1, bonePointTr1);
	  mat.multVecMatrix(bonePoint2, bonePointTr2);
	  float pt1X, pt1Y, pt1Z, pt2X, pt2Y, pt2Z;

	  pt1X = bonePointTr1[0];
	  pt1Y = bonePointTr1[1];
	  pt1Z = bonePointTr1[2];
	  pt2X = bonePointTr2[0];
	  pt2Y = bonePointTr2[1];
	  pt2Z = bonePointTr2[2];
	  
	  cubicalBox cboxTr(point((pt1X-27), (pt1Z-458), (pt1Y+33)), point((pt2X-27), (pt2Z-458), (pt2Y+33)), maxDT);
	  cboxTr.set12Lines();
     
	  /*-----------------Transformed--------------------------*/	       

	  for(float z = -100.0; z < 500.0 ; z = z + 0.5)
	    {
	      if(cboxTr.isIntersectingZPlane(z))
		{  
		  vector<point> vlist;
		  vector<point> sortpts;
		  vlist = cboxTr.getIntersectingVerticesWithZPlane(z);
		  sortpts = sortPoints(vlist);
		  //vlist = cbox.getIntersectingVerticesWithZPlane(z);
	       
		  glBegin(GL_POLYGON);
		  for(int i = 0; i<sortpts.size(); i++)
		    {
		      float X = sortpts[i].getX();
		      float Y = sortpts[i].getY();
		      float Z = sortpts[i].getZ();
		      
		      SbVec3f Vtr((X+27), (Z-33), (Y+458));
		      SbVec3f Tex;
		      inv.multVecMatrix(Vtr, Tex);
		      
		      float Xtex = (Tex[0]-27);
		      float Ytex = (Tex[2]-458);
		      float Ztex = (Tex[1]+33);
		      //glColor4f(0.5, 0.5, 0.5, 0.0);
		      //glVertex3f(170+X, Y, Z); 	   
		      //glTexCoord3f(X/256, Y/512, Z/128); glVertex3f(160+X, 512-Y, Z); 
		      glTexCoord3f(Xtex/256, Ytex/512, Ztex/256); glVertex3f(X+270, Y+460, -Z); 
		    }
		  glEnd();
		  glFlush();

		}
	    }

	}

    
    } 

}


void Reconstruct::GrowSphere(int x, int y, int z, int DT)
{
  
  //  cout<<"Reading Points"<<endl;
  // cout<<x<<" "<<y<<" "<<z<<" "<<DT<<endl;

  //parameters: center of the sphere, distance transform (radius in 3-4-5)
  int zPlane;
  int zMax, zMin;
  int radius = ToGridDistance(DT); //since DT is in 3-4-5
  zMax = radius;
  zMin = -radius;
  centerx = x;
  centery = y;
  centerz = z;
  
  for(zPlane=zMin; zPlane<= zMax; zPlane++)
    {
      GrowZCircle(x, y, z, zPlane, DT);
    }

}

 
void Reconstruct::GrowZCircle(int x, int y, int z, int zPlane, int DT)
{
  
  //parameters: center of sphere, z-center of current circle, DT
  int radius = GetYRadius(zPlane, DT);
  if(radius < 0)
    {
      cout<<"Error - could not figure out correct distance"<<endl;
      cout<<"Center: ["<<x<<", "<<y<<", "<<z<<"], DT = "<<DT<<", zPlane = "<<zPlane<<endl;
      exit(0);
    }

  int yMin, yMax, yLine;
  yMax = radius;
  yMin = -radius;
  
  for(yLine=yMin; yLine<=yMax; yLine++)
    {
        GrowYLine(x, y, z, zPlane, yLine, DT);
    }
  


}




int Reconstruct::GetYRadius(int z, int DT)
{
  int radius;
  z = abs(z);

  //Assume x=0 to find the range of y-values.
  //Assume x<y<z
  //Since x=0, DT = y*4 + (z-y)*3 = 3z + y
  radius = (DT - z*3);
  
  if(radius>z)
    {
      //Clearly against the assumption- take case x<z<y
      //equation becomes DT = z*4 + (y-z)*3 = z + 3y
      radius = (DT-z)/3;
      if((radius<0) || (radius<z))
	{
	  radius = (-1);
	}
    }
  return radius;
}


void Reconstruct::GrowYLine(int x, int y, int z, int zPlane, int yLine, int DT)
{
  //parameters: GrowCircle parameters + y-center of line to be filled in 
  //the chosen circle.
  
  int radius = GetXRadius(yLine, zPlane, DT);

  if(radius<0)
    {
      cout<<"Failed to determine proper distance"<<endl;
      cout<<"Center: ["<<x<<", "<<y<<", "<<z<<"], zPlane: "<<zPlane<<", yLine: "<<yLine<<", DT = "<<DT<<endl;
    }

  if(radius == 0)
    {
      return;
    }

  int xScan, yScan, zScan, xStart, xStop;
  yScan = y + yLine;
  zScan = z + zPlane;
  xStart = x - radius;
  xStop = x + radius;
  int index;
  
  //----------------------------------
  
  
  //----------------------------------

  for(xScan=xStart; xScan<=xStop; xScan++)
    {
      index = IndexOf(xScan-xMin, yScan-yMin, zScan-zMin);
      
      if((index>=0) && (index<volumeSize))
	{
	  int proximity = GetDistance(centerx, centery, centerz, xScan, yScan, zScan);
	  if(theProximityBuffer[index] > proximity)
	    {

	  /*     //GL vertex calls start here */
	     glBegin(GL_POINTS);
	     glVertexAttrib3fNV(3, 0.0, 1.0, 1.0);
	     glVertexAttrib3fNV(0, xScan, yScan, zScan);
	     glVertexAttrib3fNV(8, xScan/xLookupSize, yScan/yLookupSize, zScan/zLookupSize);
	    /*  glTexCoord3f(xScan/xLookupSize, yScan/yLookupSize, zScan/zLookupSize); */
/* 	     glVertex3f(xScan, yScan, zScan); */
       	     glEnd();
             glFlush();
	   
	     
   GLubyte pixels[3];
   glReadPixels((int)xScan, (int)yScan, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixels);


/*   cout<<"RED "<<(float)pixels[0]<<endl; */
/*   cout<<"GREEN "<<(float)pixels[1]<<endl; */
/*   cout<<"BLUE "<<(float)pixels[2]<<endl; */
 	      
#ifdef SAMPLED
	      theBuffer[index] = (char)(pixels[1]);
#endif
	      theProximityBuffer[index] = proximity;
	    }
	}
      else
	{
	  cout<<"Error - invalid index requested: ["<<xScan<<", "<<yScan<<", "<<zScan<<"]"<<endl;
	}
    }
	    
}


int Reconstruct::GetDistance(int x1, int y1, int z1, int x2, int y2, int z2)
{
  int diff1, diff2, diff3, retValue;
  int first, second, third;
  
  
  diff1 = (x1<x2) ? (x2-x1) : (x1-x2);
  diff2 = (y1<y2) ? (y2-y1) : (y1-y2);
  diff3 = (z1<z2) ? (z2-z1) : (z1-z2);

  if(diff1<diff2)
    {
      if(diff3<diff1)
	{
	  first = diff3;
	  second = diff1;
	  third = diff2;
	}
      else if(diff3<diff2)
	{
	  first = diff1;
	  second = diff3;
	  third = diff2;
	}
      else
	{
	  first = diff1;
	  second = diff2;
	  third = diff3;
	}
    }
  else
    {
      if(diff3<diff2)
	{
	  first = diff3;
	  second = diff2;
	  third = diff1;
	}
      else if(diff3<diff1)
	{
	  first = diff2;
	  second = diff3;
	  third = diff1;
	}
      else
	{
	  first = diff2;
	  second = diff1;
	  third = diff3;
	}
    }

  retValue = first*5 + (second-first)*4 + (third-second)*3;
  return retValue;
      
}

long Reconstruct::IndexOf(int x, int y, int z)
{
  if((x<0) || (y<0) || (z<0))
    {
      cout<<"Error - negative index requested: ["<<x<<", "<<y<<", "<<z<<"]"<<endl;
    }
  
  return (z*xSize*ySize + y*xSize + x);
}

int Reconstruct::GetXRadius(int y, int z, int DT)
{
  int radius;

  y = abs(y);
  z = abs(z);
  
  if(y<z)
    {
      //y<z : so it's one of x<y<z or y<x<z or y<z<x
      //case 1A : x<y<z
      radius = DT - y - 3*z;
      if((radius<0) || (radius>y))
	{
	  //FAILED: case 1B: y<x<z
	  radius = DT - y - 3*z;
	  
	  if((radius<y) || (radius>z))
	    {
	      //FAILED: case 1C: y<z<x
	      radius = (DT-y-z)/3;
	      if(radius<z)
		{
		  //TOTAL FAILURE
		  radius = (-1);
		}
	    }
	}
    }
  else
    {
      //z<y: so it's one of x<z<y or z<x<y or z<y<x
      //case 2A: x<z<y
      radius = DT - z - 3*y;

      if((radius>z) || (radius<0))
	{
	  //FAILED: case 2B: z<x<y
	  radius = DT - z - 3*y;
	  
	  if((radius<z) || (radius>y))
	    {
	      //FAILED: case 2C: z<y<x
	      radius = (DT-z-y)/3;
	      if(radius<y)
		{
		  //TOTAL FAILURE
		  radius = (-1);
		}
	    }
	}
    }

  return radius;

}

int Reconstruct::ToGridDistance(int DT)
{
  return (DT/3);
}


