#ifndef LOADPARAMS_H
#define LOADPARAMS_H

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <GL/gl.h>
#include <GL/glut.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/SoDB.h>
#include <Inventor/SbLinear.h>
#include <Inventor/SbViewportRegion.h>
#include <qapplication.h>
#include <qgl.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qpopupmenu.h>

#include "QtInterface.h"


#define SAMPLED
#define PADDING 5
#define PI 3.1415926535897
#define SQRT2 1.4142136
#define ID_QUIT 666
#define ID_SKELETON 777



unsigned char* loadVolume(char*, int, int, int);
void initGL(unsigned char*, int, int, int);
static FILE* tskelFile;

class point 
{
 private:
  float x, y, z;

 public:
  //Constructors
  point() { x=0; y=0; z=0; }
  point(float _x, float _y, float _z) { x=_x; y=_y; z=_z; }

  //assignment operators
  void setXYZ(float _x, float _y, float _z) { x=_x; y=_y; z=_z; }
  void setPoint(point pnew) { x = pnew.x; y = pnew.y; z = pnew.z; }

  //access operators
  float getX() { return x; }
  float getY() { return y; }
  float getZ() { return z; }
  point getPoint() { return point(x, y, z); }

  //overload input and output operators.
  friend istream& operator >> (istream& is, point& p)
    {
      is >> p.x; is >> p.y; is >> p.z;
      return is;
    }
  
  friend ostream& operator << (ostream& os, point& p)
    {
      os << " " << p.x <<" "<< p.y << " " << p.z << " ";
      return os;
    }

  //other functions

  bool operator < (point const& p) const
    {
      if(x<p.x) { return true; }
      if(x==p.x)
	if(y<p.y) return true;
      if(x>p.x) { return false; }
    }
  
  bool operator == (point const& p) const
    {
      if(x == p.x && y == p.y && z == p.z)
	return true;
      else 
	return false;
    }
  
  bool operator != (point const& p) const
    {
      if(x != p.x || y != p.y || z != p.z)
	return true;
      else 
	return false;
    }
  
  //overload arithematic operators
  point operator + (point const& p) const
    {
      return point(x+p.x, y+p.y, z+p.z);
    }
  
  point operator - (point const& p) const
    {
      return point(x-p.x, y-p.y, z-p.z);
    }

  point& operator = (point const& p)
    {
      x=p.x; y=p.y; z=p.z;
      return (*this);
    }

};


class line
{
 private:
  point p1, p2;

 public:
  
  //Constructors
  line() { p1 = point(0, 0, 0); p2 = point(0, 0, 0); }
  line(point _p1, point _p2) { p1 = _p1; p2 = _p2; }

  //assignment operators
  void setP1P2(point _p1, point _p2) { p1 = _p1; p2 = _p2; }
  void setLine(line lnew) { p1 = lnew.p1; p2 = lnew.p2; }

  //access operators
  point getP1() { return p1; }
  point getP2() { return p2; }

  //other functions
  bool intersectZPlane(float zPlane)
    {
      float d;
      d = (zPlane - p1.getZ())/(p2.getZ() - p1.getZ());
      if(d>=0 && d<=1)
	{
	return true;
	}
      else return false;
    }

  //intersection point with Z plane
  point intersectPoint(float zPlane)
    {
      float x, y, z, d;
      d = (zPlane-p1.getZ())/(p2.getZ() - p1.getZ());
      
      x = (1-d)*p1.getX() + d*p2.getX();
      y = (1-d)*p1.getY() + d*p2.getY();
      z = (1-d)*p1.getZ() + d*p2.getZ();
      
      return point(x, y, z);
    }
 
  //get point on line segment at d(fraction) from p1
  point getPointOnLineFromP1(float d)
    {
      return point((1-d)*p1.getX()+d*p2.getX(), (1-d)*p1.getY()+d*p2.getY(), (1-d)*p1.getZ()+d*p2.getZ());
    }

  //get point on line segment at d(fraction) from p2;
  point getPointOnLineFromP2(float d)
    {
      return point((1-d)*p2.getX()+d*p1.getX(), (1-d)*p2.getY()+d*p1.getY(), (1-d)*p2.getZ()+d*p1.getZ());
    }


  //distance of an arbitrary point from this line
  float lineTo3dpointDistance(point px)
    {
      SbVec3f p1p2((p2.getX()-p1.getX()), (p2.getY()-p1.getY()), (p2.getZ()-p1.getZ()));
      SbVec3f p1px((px.getX()-p1.getX()), (px.getY()-p1.getY()), (px.getZ()-p1.getZ()));
      SbVec3f p1p2_cross_p1px;
      p1p2_cross_p1px = p1p2.cross(p1px);

      return (float)(p1p2_cross_p1px.length()/p1p2.length());
    }
      
  //overloading = operator
  line& operator = (line const& l)
    {
      p1 = l.p1; p2 = l.p2;
      return (*this);
    }

  //overloading stream operators

  friend istream& operator >> (istream& is, line l)
    {
      is >> l.p1; is >> l.p2;
      return is;
    }
  
  friend ostream& operator << (ostream& os, line l)
    {
      os<<"P1 = "<<l.p1<<" : P2 = "<<l.p2<<endl;
      return os;
    }
  

};

class cubicalBox
{
 public:
  line l1;
  float maxDT;
  vector<line> vline;

 public:
  //Constructors
  cubicalBox() { l1 = line(point(0, 0, 0), point(0, 0, 0)); }
  cubicalBox(point _p1, point _p2, float _maxDT)
    {
      l1 = line(_p1, _p2); 
      maxDT = _maxDT;
    }
  cubicalBox(line lnew, float _maxDT) { l1.setLine(lnew); maxDT = _maxDT; }

  //assignment operators
  void setLinePoints(point _p1, point _p2) { l1.setP1P2(_p1, _p2); }
  void setCenterLine(line lnew) { l1.setLine(lnew); }
  void setmaxDT(float _maxDT) { maxDT = _maxDT; }
  void setcubicalBox(point _p1, point _p2, float _maxDT) { l1.setP1P2(_p1, _p2); maxDT = _maxDT; }
  void set12Lines();

  //access operators
  line getCenterLine() { return l1; }
  float getmaxDT() { return maxDT; }
  vector<line> get12Lines() { return vline; }

  //intersecting vertices with zPlane
  bool isIntersectingZPlane(float );
  vector<point> getIntersectingVerticesWithZPlane(float );
 
};

class thetaIndex
{
 private:
  float theta;
  int index;
  
 public:
  //Constructors
  thetaIndex() { theta = 0; index = 0; }
  thetaIndex(float _theta, int _index) { theta = _theta; index = _index; }

  //assignment operators
  void setTheta(float _theta) { theta = _theta; }
  void setIndex(int _index) { index = _index; }
  void setThetaIndex(float _theta, int _index) { theta = _theta; index = _index; }

  //access operators
  float getTheta() { return theta; }
  int getIndex() { return index; }

  //overloading < operator for sorting purposes
  bool operator < (thetaIndex const& tid) const
    {
      if(theta < tid.theta) return true;
      else return false;
    }
};


class Vertex
{
 private:
  point poscoords;
  point texcoords;
  
 public:
  //Constructors
  Vertex() { poscoords.setXYZ(0, 0, 0); texcoords.setXYZ(0, 0, 0); }
  Vertex(point _poscoords, point _texcoords)
    {      poscoords = _poscoords; texcoords = _texcoords; }

  //Assignment operators
  void setPosCoords(point _poscoords) { poscoords = _poscoords; }
  void setTexCoords(point _texcoords) { texcoords = _texcoords; }
  void setVertex(point _poscoords, point _texcoords)
    { poscoords = _poscoords; texcoords = _texcoords; }

  //Access operators
  point getPosCoords() { return poscoords; }
  point getTexCoords() { return texcoords; }

  //overloading operators
  friend ostream& operator << (ostream& os, Vertex v)
    {
      os<<"Vertex Pos. "<<v.poscoords<<" Tex Coords "<<v.texcoords<<endl;
      return os;
    }

};


vector<point> sortPoints(vector<point>);
vector<int> sortVertices(vector<Vertex>);

class connectLines
{
 private:
  int numpoints;
  int p1Index, p2Index;

 public:
  //Constructors
  connectLines() { numpoints = 2; p1Index =0; p2Index = 0; }
  connectLines(vector<point> vpt, point p1, point p2)
    {
      numpoints = 2;
      for(int i = 0; i < vpt.size(); i++)
	{
	  if(vpt[i] == p1)
	    p1Index = i;
	  if(vpt[i] == p2)
	    p2Index = i;
	}
    }

  connectLines(int _numpoints, int _p1Index, int _p2Index)
    { 
      numpoints = _numpoints;
      p1Index = _p1Index;
      p2Index = _p2Index;
    }

  //access operators
  int getNumpoints() { return numpoints; }
  int getp1Index() { return p1Index; }
  int getp2Index() { return p2Index; }

  //operator overloading
  friend ostream& operator << (ostream& os, connectLines cL)
    {
      os<<cL.numpoints<<" "<<cL.p1Index<<" "<<cL.p2Index<<" ";
      return os;
    }

};


class jointGeometry
{
 private:
  int jointIndex;
  vector<int> connectedBoneIdx;
  SbVec3f v1, v2;
  vector<point> midPlane;
  vector<point> Plane1;
  vector<point> Plane2;
  float v1Mag, v2Mag;
  
 public:
  //Constructor
  jointGeometry() { jointIndex = 0; }
  jointGeometry(int _jointIndex, vector<int> _connectedBoneIdx)
    { 
      jointIndex = _jointIndex;
      connectedBoneIdx = _connectedBoneIdx;
    }

  //Assignment operators
  void setJointGeometry(int _jointIndex, int _connectedBoneIdx)
    {
      jointIndex = _jointIndex;
      connectedBoneIdx.push_back(_connectedBoneIdx);
    }

  void setV1(SbVec3f _v1) { v1 = _v1; }
  void setV2(SbVec3f _v2) { v2 = _v2; }
  void setMidPlane(point _p) { midPlane.push_back(_p); }
  void setPlane1(point _p ) { Plane1.push_back(_p); }
  void setPlane2(point _p) { Plane2.push_back(_p); }
  void setv1Mag(float _v1Mag) { v1Mag = _v1Mag; }
  void setv2Mag(float _v2Mag) { v2Mag = _v2Mag; }

  //Access operators
  int getJointIndex() { return jointIndex; }
  vector<int> getConnectedBoneIdx() { return connectedBoneIdx; }
  SbVec3f getV1() { return v1; }
  SbVec3f getV2() { return v2; }
  int getNumberOfConnectedBones() { return connectedBoneIdx.size(); }
  vector<point> getMidPlane() { return midPlane; }
  vector<point> getPlane1() { return Plane1; }
  vector<point> getPlane2() { return Plane2; }
  float getv1Mag() { return v1Mag; }
  float getv2Mag() { return v2Mag; }
  
  //overloading operators
  friend ostream& operator << (ostream& os, jointGeometry jGeom)
    {
      cout<<"jointIndex = "<<jGeom.jointIndex<<" :  connectedBoneIndices ";
      for(int i = 0; i < jGeom.connectedBoneIdx.size(); i++)
	{ cout<<jGeom.connectedBoneIdx[i]<<" "; }    
      return os;
    }
};



/* class skeletonFile : public QGLWidget */
/* { */
/*   Q_OBJECT */

/*  public: */
/*   skeletonFile(QWidget *parent, const char *name) */
/*     : QGLWidget( parent, name) {} */

/*   ~skeletonFile() { makeCurrent(); } */
/*   skeletonFile() {} */

/*  private: */
/*   vector<point> vpt; */
/*   vector<connectLines> vcL; */
/*   vector<float> vMaxDis; */
/*   vector<SbMatrix> vecmat; */
/*   vector<jointGeometry> vGeom; */
  
/*  public: */
  
 



/*   //Access operators */
/*   vector<point> getpointList() { return vpt; } */
/*   vector<connectLines> getlineList() { return vcL; } */
/*   vector<float> getMaxDisList() { return vMaxDis; } */
/*   vector<SbMatrix> getTransformationList() { return vecmat; } */

/*   //SLOTS */
  
/*   public slots: */
/*   void setXRotation(int degrees); */
/*   void setYRotation(int degrees); */
/*   void setZRotation(int degrees); */
  
/*  protected: */
/*   void initializeGL(void);    */
/*   void paintGL(void); */
/*   void SetSkeleton(FILE*); */
/*   void renderCuboids(void); */
/*   void renderJointSpheres(bool, float); */
/*   void defineJointGeometry(void); */
/*   void renderJointGeometry(void); */

/* }; */


class Reconstruct
{
 public:
  int xSize, ySize, zSize;
  int xLookupSize, yLookupSize, zLookupSize;
  int xMin, yMin, zMin, xMax, yMax, zMax;
  int centerx, centery, centerz;

  FILE* tskelFile;
  FILE* inFile;
  FILE* outFile;
  FILE* tskelFileList;

  unsigned char* theBuffer;
  unsigned char* theLookupBuffer;
  unsigned char* theProximityBuffer;

  long volumeSize;
  long lookupVolumeSize;
  
  SbMatrix mat, inv;

  //the below are required for sampled lookup only
  
  float X1, X2, xLookup;
  float Y1, Y2, yLookup;
  float Z1, Z2, zLookup;
  float RATIO;
  float ONE_BY_EXTENT;

 public:
  void GrowZCircle(int x, int y, int z, int zPlane, int DT);
  void GrowYLine(int x, int y, int z, int zPlane, int yLine, int DT);
  int GetDistance(int x1, int y1, int z1, int x2, int y2, int z2);
  int ToGridDistance(int DT);
  int To345Distance(int radius);
  int GetXRadius(int y, int z, int DT);
  int GetYRadius(int z, int DT);
  long IndexOf(int x, int y, int z);
  long LookupIndexOf(int x, int y, int z);
  void Execute(int argc, char* argv[]);
  void TransformedRecon();
  void Init();
  int SampledLookup(int xScan, int xStart);
  void GrowSphere(int x, int y, int z, int DT);
};
  

#endif //LOADPARAMS_H

