#include <stdlib.h>
#include <stdio.h>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>
#include "QtInterface.h"
     
using namespace std;
static GLuint texName;
static GLuint VPID;
GLfloat xangle=0.0f, yangle=0.0f;
float Xver, Yver, Zver;

//Mouse
int mousePos[2] = {0, 0};
bool mouseDown[2] = {false, false};

//Some Camera Stuff
float objectRotation[3] = {0.0, 0.0, 0.0};


const unsigned char VertexProgram[] =
  {
    //Standard Vertex Program header
    "!!VP1.0"
    
    //These 4 commands will transform the vertex by the modelview
    //matrix. We are multiplying by the tracked matrix in registers
    //0-3. Note that R0 is a temporary register while v is the INPUT
    // vertex. 
    
    "DP4 R0.x, c[0], v[OPOS];"
    "DP4 R0.y, c[1], v[OPOS];"
    "DP4 R0.z, c[2], v[OPOS];"
    "DP4 R0.w, c[3], v[OPOS];"

    //These 4 commands will now project the vertex into screen space
    //We are multiplying by the projection matrix stored in registers
    //4-7. Note that o[HPOS] is the final output position register.
    
    "DP4 o[HPOS].x, R0, c[4];"
    "DP4 o[HPOS].y, R0, c[5];"
    "DP4 o[HPOS].z, R0, c[6];"
    "DP4 o[HPOS].w, R0, c[7];"
    
    //Tranform texture coordinates
/*     "MUL o[TEX0], v[TEX0], c[8];" */
/*     "MUL o[TEX0], v[TEX0], c[9];" */
/*     "MUL o[TEX0], v[TEX0], c[10];" */
/*     "MUL o[TEX0], v[TEX0], c[11];" */
    "MOV o[COL0], v[COL0];"
    "MOV o[TEX0], v[TEX0];"
    
    //Finish off our program with the END command
    
    "END"
  };


void SetupVertexProgram(void)
{ 
  //Transformation matrix
  GLfloat transMat[16] = { 1, 5, 9,  13,
			   2, 6, 10, 14,
			   3, 7, 11, 15,
			   4, 8, 12, 16};

  //Enable the vertex program
  //  glEnable(GL_VERTEX_PROGRAM_NV);


    //Initializaing vertex program
    glGenProgramsNV(1, &VPID);

    //Bind the vertex program
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, VPID);

    //Load the vertex program
    glLoadProgramNV(GL_VERTEX_PROGRAM_NV, VPID, strlen((char*)VertexProgram), (unsigned char*)VertexProgram);

     //Error checking for vertex programs. This will return 
     //GL_INVALID_OPERATION if the vertex program has any syntax errors.

    if(glGetError() == GL_INVALID_OPERATION)
      {
      cout << "Error: glLoadProgramNV has returned an invalid operation" << endl;
      exit(1);
      }
    
     //Place matrices into program registers
     //(enum, starting register, type of matrix, operation on matrix)
     
     //This will place the modelview matrix into the first 4 registers(0-3)
     glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW, GL_IDENTITY_NV);

     //This will place the projection matrix into the next 4 registers(4-7)
     glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_PROJECTION, GL_IDENTITY_NV);
     
     //Track Transformation matrix in registers

      glMatrixMode(GL_MATRIX0_NV);
      glLoadMatrixf(transMat);
      glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 8, GL_MATRIX0_NV, GL_IDENTITY_NV);


     //Enable vertex arrays
     glEnableClientState(GL_VERTEX_ARRAY);
     cout<<"vertex program setup"<<endl;
     
}
    
  
void skeletonFile::initializeGL(void)
{  
  unsigned char* vol;
  vol = new unsigned char[256*512*256*sizeof(unsigned char)];
  ifstream in("halfcrop.256x512x256.vol");
  in.read(vol, 256*512*256*sizeof(unsigned char));
 
   
   if (!glutExtensionSupported("GL_EXT_paletted_texture")) {
      printf("Sorry, GL_EXT_paletted_texture not supported\n");
      exit(0);
   }

   /* load the colormap table for each texel-index */
   FILE* colormap;
   colormap = fopen("good.icol", "r");
   GLubyte table[256][4];
   char str[256];
   int idx, r, g, b;
   fscanf(colormap, "%[^\n]", str);
   
   while(!feof(colormap))
     {
       if(sscanf(str, "%d%d%d%d", &idx, &r, &g, &b) < 4)
	 {
	   cerr<<"Error in colormap file\n"<<endl;
	   cout<<":- "<<str<<endl;
	 }
       table[idx][0]= r;
       table[idx][1]= g; 
       table[idx][2]= b;
       table[idx][3]= 2;
       
       //cout<<idx<<" "<<r<<" "<<g<<" "<<b<<endl;
       fscanf(colormap, "\n");
       fscanf(colormap, "%[^\n]", str);

     }
  

 //Select clearing color
  glClearColor(0.2, 0.2, 0.3, 0.4);

  glEnable(GL_TEXTURE_3D);
  //   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   //Initialize viewing values
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   //glOrtho(-100.0, 680.0, -200.0, 480.0, -5000.0, 5000.0);
   glOrtho(-360.0, 360.0, -300.0, 300.0, -5000.0, 5000.0);
   //glOrtho(-500.0, 500.0, -500.0, 500.0, -5000.0, 5000.0);
   
   //glFrustum(-500.0, 500.0, -500.0, 500.0, -500.0, 500.0);
   //gluPerspective(75.0f, 1.0f, -500.0f, 500.0f);
   glMatrixMode(GL_MODELVIEW);
   
  cout<<"Reaching init"<<endl;

  glShadeModel(GL_FLAT);
  glEnable(GL_DEPTH_TEST); 
  glDepthFunc(GL_ALWAYS);

  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

  glGenTextures(1, &texName);
  glBindTexture(GL_TEXTURE_3D, texName);
  
  glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_R, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
  
  glColorTableEXT(GL_TEXTURE_3D, GL_RGBA, 256, GL_RGBA, GL_UNSIGNED_BYTE, table);
  cout<<"Loading Texture"<<endl;
  glTexImage3D(GL_TEXTURE_3D, 0, GL_COLOR_INDEX8_EXT, 256, 512, 256, 0, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, vol);   
  //glTexImage3D(GL_TEXTURE_3D, 0, 1, 256, 512, 256, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, vol);

 if(glGetError()== GL_INVALID_VALUE)
    {
    cout<<"A NUMERIC CONSTANT IS OUT OF RANGE!"<<endl;
    }
 
  // Load up the Vertex Program
  cout<<"Loading up Vertex Program..."<<endl;
 
  SetupVertexProgram();
  this->SetSkeleton();
  cout<<"passed"<<endl; 
}

  
void skeletonFile::paintGL(void)
{ 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  //gluLookAt(0.0, 0.0, 30.0, 0.0, 0.0, 0.0, 0.0, -1.0, 0.0);
  glColor4f(0.2, 0.2, 0.3, 0.0055);
  //glColor4f(1.0, 1.0, 1.0, 1.0);
  
  /************TESTING AREA***************/
  
  //float modelview[16];
  /*
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
  cout<<"ModelView matrix"<<endl;
  for(int i=0; i<4; i++)
    {
      for(int j=0; j<4; j++)
	cout<<modelview[4*i+j]<<" ";
      cout<<endl;
    }

  */
  /************TESTING-ENDS***************/


  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  glBindTexture(GL_TEXTURE_3D, texName);
   
  glDisable(GL_VERTEX_PROGRAM_NV);
  glBindProgramNV(GL_VERTEX_PROGRAM_NV, VPID);
  
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
  glBlendEquation(GL_FUNC_ADD);

    GLboolean residences;
    if(glAreTexturesResident(1, &texName, &residences))
      { 
	cout<<"Texture Successfully Loaded ...."<<endl;
      
      }	     
    
    if(glIsEnabled(GL_VERTEX_PROGRAM_NV))
      cout<<"VP Program Enabled"<<endl;
    else
      cout<<"VP Program Disabled"<<endl;
    
   
    renderCuboids();
    renderJointGeometry();
  
    
    //Reconstruct theObject;
    //theObject.Execute(ARGC, ARGV);

/*   glDisable(GL_VERTEX_PROGRAM_NV); */
/*   glutSwapBuffers(); */
 

/*  //glDrawBuffer(GL_BACK_LEFT); */
/*   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT); */
 
/*   ofstream out("image.ppm"); */
/*   unsigned char* image; */
/*   image = new unsigned char[3*500*500*sizeof(unsigned char)]; */
/*   glReadBuffer(GL_BACK_LEFT); */
/*   glReadPixels(0, 0, 500, 500, GL_RGB, GL_UNSIGNED_BYTE, image); */
/*   out<< "P6\n" << "500 500\n255"<<endl; */
  //out.write(image, 3*500*500*sizeof(unsigned char));
  
/*   for(int j = 499; j >= 0; j--) */
/*     for(int i = 0; i < 500; i++) */
/*       { */
/* 	out<<image[3*j*500 + 3*i + 0]<<image[3*j*500 + 3*i + 1]<<image[3*j*500 + 3*i + 2]; */
/* 	//cout<<(int)image[3*j*500 + 3*i + 0]<<" "<<(int)image[3*j*500 + 3*i + 1]<<" "<<(int)image[3*j*500 + 3*i + 2]<<endl; */
/*       } */
  
/*   cout<<"Dumping Image on disk..."<<endl; */
 
    
}

void noDisplay(void)
{
  glClearColor(0.5, 0.5, 0.5, 1.0);
  glColor3f(0.5, 0.5, 0.5);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glutSwapBuffers();
  
}

void keyboard(unsigned char key, int x, int y)
{

  switch (key) {

  case 27: 
    exit(0);
    break;

  case 'x':
    exit(-1);
    break;

  case 'f':        
    glutFullScreen();
    break;

  default:
    break;
  }
}

void MouseMove(int x, int y)
{
  int posdiff[2];
  
  //Calculate the change in mouse position
  posdiff[0] = mousePos[0] - x;
  posdiff[1] = mousePos[1] - y;

  //Store the new mouse position
  mousePos[0] = x;
  mousePos[1] = y;

  //If the left mouse button is held down, rotate
  if(mouseDown[0])
    {
      objectRotation[0] -= posdiff[1];
      objectRotation[1] -= posdiff[0];
    } 
  glutPostRedisplay();
}

void MouseClick(int button, int down, int x, int y)
{
  //Update the mouse states
  switch(button)
    { 
    case GLUT_LEFT_BUTTON:
      if(down == GLUT_DOWN)
	mouseDown[0] = true;
      else
	mouseDown[0] = false;
      break;

    case GLUT_RIGHT_BUTTON:
      if(down == GLUT_DOWN)
	mouseDown[1] = true;
      else
	mouseDown[1] = false;
      break;
    }
  mousePos[0] = x;
  mousePos[1] = y;
}

void IdleFunc()
{
  glutPostRedisplay();
}


void createMenu(int value)
{
  switch (value)
    {
    case ID_QUIT:
      exit(0);
      break;
    case ID_SKELETON:
      glutSetWindow(2);
      glClear(GL_COLOR_BUFFER_BIT);
      glutSwapBuffers();
      cout<<glutGetWindow()<<endl;
      cout<<"View skeleton"<<endl;
      break;
    default:
      break;
    }

}



int main(int argc, char** argv)
{
   
  if (argc < 13)
    {
      cerr<<endl<<"Hardware Volume Reconstruction"<<endl;
      cerr<<"Usage: samreconRen <tskelfile> <origObject> <outObject> <xSize> <ySize> <zSize> [xMin] [xMax] [yMin] [yMax] [zMin] [zMax]"<<endl;
      exit(-1);
    }
 
  
  //vol = loadVolume(argv[2], atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
  
  //FILE* tskelFile;
  tskelFile = fopen(argv[1], "r");
  
 

  QApplication::setColorSpec( QApplication::CustomColor );
  QApplication app(argc, argv);
 
  if ( !QGLFormat::hasOpenGL() )
    {
      qWarning("This system has no OpenGL support. Exiting...!");
      return -1;
    }
  
  
  QtInterface interface;
  app.setMainWidget( &interface );
  interface.show();
  
  int ret = app.exec();
  return ret;
  
  /*   ARGC = argc; */
  /*   ARGV = argv; */
  
  /*---------Initializing----------*/
  

/*   glutInit(&argc, argv); */
/*   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE); */
/*   glutInitWindowSize(500, 500); */
/*   glutInitWindowPosition(100, 100); */
/*   glutCreateWindow(argv[0]); */
/*   glutDisplayFunc(display);   */
/*   glutKeyboardFunc(keyboard);  */
/*   glutMotionFunc(MouseMove); */
/*   glutMouseFunc(MouseClick); */
  
/*   initGL(vol, atoi(argv[4]), atoi(argv[5]), atoi(argv[6])); */
/*   display(); */
  /*--------Initialization Complete----------*/
 
 

  //glutMainLoop();
  exit(-1);
  return 0;
}



  
      
