#include <stdio.h>
#include <ctype.h>

#include "skelView.h"

// #include "vtkConeSource.h"
// #include "vtkPolyDataMapper.h"
// #include "vtkActor.h"

// #include <qapplication.h>

skelView::skelView(QWidget *parent = 0, const char *name = 0)
	: RenderWindowInteractor(parent, name) {

	InitStructure();
	InitState();

	strcpy(cInputFileName, cInputFN);
	strcpy(cOutputFileName, cOutputFN);	

	setFocusPolicy(QWidget::StrongFocus);
}
	
skelView::skelView( QGLFormat glf, QWidget *parent = 0, const char *name = 0)
	: RenderWindowInteractor(glf, parent, name) {

	InitStructure();
	InitState();

	strcpy(cInputFileName, cInputFN);
	strcpy(cOutputFileName, cOutputFN);	
	
	setFocusPolicy(QWidget::StrongFocus);
}


skelView::~skelView() {
	Cleanup();
};

bool skelView::InitStructure() {
	// The renderer, renderwindow and renderwindowinteractor
	pRen = this->GetRenderer();
	pRenWin = this->GetRenderWindow();
	pPicker = vtkCellPicker::New();
	
	// Selected and Unselected properties
	pSelectedProperties = vtkProperty::New();
	pNormalProperties = vtkProperty::New();

	// the assembly
	ptheAssembly = vtkAssembly::New();
	
	if((pRen == NULL) || (pRenWin == NULL) || (pPicker == NULL) || \
		(pSelectedProperties == NULL) || (pNormalProperties == NULL) || \
		(ptheAssembly == NULL)) {
		cout << "Error initializing application !\n";
		return false;
	}

	pPicker->SetTolerance(0.01);
	pRen->SetBackground(0.2, 0.2, 0.3);
		
	SetNormalProperties();
	SetSelectedProperties();

	nNrOfPoints = 0;
	nNrOfActors = 0;

	pSelectedAssembly = NULL;
	pSelectedActor = NULL;

	return true;
}


bool skelView::InitState() {
	if (ReadPointsLinesAndTransformations()) {
		cout << "Done reading input file\n";
		if(MakeEachLineAnActor()) {
			cout << "Made each line an actor.\n";
			if(BuildActorHierarchy()) {
	
				cout << "Biult actor hierachy.\n";		
				pRen->AddActor(ptheAssembly);
				pRen->ResetCamera();

				return true;
			}
		}
	}
	return false;
}

void skelView::Cleanup() {
	pSelectedAssembly = NULL;
	pSelectedActor = NULL;

	if(pPicker != NULL) pPicker->Delete();

	//pRenWin->Delete();
	//pRen->Delete();
	if (pSelectedProperties != NULL) pSelectedProperties->Delete();
	if(pNormalProperties != NULL) pNormalProperties->Delete();
	if(ptheAssembly != NULL) ptheAssembly->Delete();

	for(int i=0; i < nNrOfActors; i++) {
		if(aAssemblies[i] != NULL) aAssemblies[i]->Delete();
		if(aActors[i] != NULL) aActors[i]->Delete();
		if(aMappers[i] != NULL) aMappers[i]->Delete();
		if(aLineSources[i] != NULL) aLineSources[i]->Delete();
	}
}


void skelView::SetNormalProperties() {
	pNormalProperties->EdgeVisibilityOn();
	pNormalProperties->SetEdgeColor(1.0, 1.0, 1.0);
	pNormalProperties->SetLineWidth(1.0);
}

void skelView::SetSelectedProperties() {
	pSelectedProperties->EdgeVisibilityOn();
	pSelectedProperties->SetEdgeColor(1.0, 0.0, 0.0);
	pSelectedProperties->SetColor(1.0, 0.0, 0.0);
	pSelectedProperties->SetLineWidth(3.0);
}

bool skelView::ReadPointsLinesAndTransformations() {
	bool retval = true;
	int section = 0;	// indicates in which array I have to write
					// the data: 1 - points, 2 - lines
	bool sectionstart = true;
	bool HadPoints = false;
	bool HadLines = false;
	char ln[201];
	float a, b, c;
	int m, n, p;
	int i;
	

	
	FILE* fh = fopen(cInputFileName, "r");
	if(fh == NULL) {
		cout << "Error opening file " << cInputFileName << " for reading !\n";
		return false;
	}
	cout << "File " << cInputFileName << " succesfuly opened for reading !\n";

	while(fgets(ln, 200, fh) != NULL) {
		// not EOF yet
		sectionstart = false;
		// delete the spaces at the begining of the line
		while(isspace(ln[0]) != 0) {
			strcpy(ln, ln+1);
		}
		//cout << ln;
		
		if(strlen(ln) > 0) {
			if(isalpha(ln[0]) != 0) {
				// the first character is a letter
				if(section == 1) HadPoints = true;
				else if(section == 2) HadLines = true;	
	
				if(strncmp(ln, "POINTS", 6) == 0) {
					section = 1;
					sectionstart = true;
				} 
				else {
					if(strncmp(ln, "LINES", 5) == 0) {
						section = 2;
						sectionstart = true;
					} 
					else {
						section = 0;
						sectionstart = true;
					}
				}
			}
			
	
			if(sectionstart) {
				//cout << "Section START";
				if(HadPoints && HadLines) {
					break;
				}
			}
			else {
				if(section == 1) {
					if(nNrOfPoints >= nMAXNrOfPoints) {
						cout << " !! Error - read more than" << nMAXNrOfPoints << " POINTS from the input file";
						return false;
					}
	
					a = 0; b = 0; c = 0;
					i = sscanf(ln, "%f %f %f", &a, &b, &c);
					if (i < 3) {
						cout << " !! Error - reading point data from line: " << ln;
						return false;
					}
					aPoints[nNrOfPoints][0] = a;
					aPoints[nNrOfPoints][1] = b;
					aPoints[nNrOfPoints][2] = c;
	 				nNrOfPoints++;
				}
				else {
					if(section == 2) {
						if(nNrOfActors >= nMAXNrOfActors) {
							cout << " !! Error - read more than" << nMAXNrOfActors << " LINES from the input file";
							return false;
						}
						m = 0; n = 0; p = 0;
						i = sscanf(ln, "%d %d %d", &m, &n, &p);
						if (i < 3) {
							cout << " !! Error - reading line data from line " << ln;
							return false;
						}
						aLines[nNrOfActors][0] = n;
						aLines[nNrOfActors][1] = p;
		 				nNrOfActors++;
					}
				}
			}	
		}
		strcpy(ln, "");
	}
	
	fclose(fh);
	cout << "Closed input file \n";

/*
	// write out  the points and the lines
	cout << "POINTS:" << nNrOfPoints << "\n";
	for(i=0;i<nNrOfPoints;i++) {
		cout << aPoints[i][0] << " " << aPoints[i][1] << " " << aPoints[i][2] << "\n";
	}

	cout << "LINES:" << nNrOfActors << "\n";
	for(i=0;i<nNrOfActors;i++) {
		cout << aLines[i][0] << " " << aLines[i][1] <<  "\n";
	}
*/
	return retval;
}



bool skelView::MakeEachLineAnActor() {
	int i = 0;
	int p1, p2;

	for(i=0;i < nNrOfActors;i++) {
		//cout << i;p
		aLineSources[i] = vtkLineSource::New();
		if(aLineSources[i] == NULL) {
			cout << "Error alocating memory for LineSources...\n";
			return false;
		}

		p1 = aLines[i][0];
		p2 = aLines[i][1];
		//cout << p1 << " " << p2;
		if((p1 >= nNrOfPoints) && (p2 >= nNrOfPoints)) {
			cout << "Error: a line is referencing a point that was not defined: line number = " << i << "\n";
			return false;
		}
		
		aLineSources[i]->SetPoint1(aPoints[p1][0], aPoints[p1][1], aPoints[p1][2]);
		aLineSources[i]->SetPoint2(aPoints[p2][0], aPoints[p2][1], aPoints[p2][2]);
		
		//cout << "Line " << i << " : " << p1 << "-" << p2;
		//cout << "    from " << aPoints[p1][0] << " " << aPoints[p1][1] << " " << aPoints[p1][2];
		//cout << "    to " << aPoints[p2][0] << " " << aPoints[p2][1] << " " << aPoints[p1][2] << "\n";
		
		aMappers[i] = vtkPolyDataMapper::New();
		if(aMappers[i] == NULL) {
			cout << "Error alocating memory for Mappers...\n";
			return false;
		}
		aMappers[i]->SetInput(aLineSources[i]->GetOutput());

		aActors[i] = vtkActor::New();
		if(aActors[i] == NULL) {
			cout << "Error alocating memory for Actors...\n";
			return false;
		}
		aActors[i]->SetMapper(aMappers[i]);

		//Encapsulate each actor into an assembly
		aAssemblies[i] = vtkAssembly::New();
		if(aAssemblies[i] == NULL) {
			cout << "Error alocating memory for Assemblies...\n";
			return false;
		}
		aAssemblies[i]->AddPart(aActors[i]);
	}
	return true;
}

bool skelView::BuildActorHierarchy() {
	
	bool aLineFlags[nMAXNrOfActors];
	
	for(int i=0; i < nNrOfActors; i++) {
		aLineFlags[i] = false;
	}
	return AddActors(ptheAssembly, 0, aLineFlags);
}

bool skelView::AddActors(vtkAssembly* prootAssembly, 				 int nPointNr, 		\
	       bool 		aLineFlags[nMAXNrOfActors]) {

	int pt2 = 0;

	// add to this assembly, all the lines that have nPointNr as one of their points
	for(int i=0; i < nNrOfActors; i++) {
		if(aLineFlags[i] == false) {
			if(LineContainsPoint(i, nPointNr)) {
				// set the added assembly's origin to nPointNr
				aAssemblies[i]->SetOrigin(aPoints[nPointNr][0], aPoints[nPointNr][1], aPoints[nPointNr][2]);

				prootAssembly->AddPart(aAssemblies[i]);
				aLineFlags[i] = true;
				pt2 = GetTheOtherPoint(i, nPointNr);
				AddActors(aAssemblies[i], pt2, aLineFlags);
			}
		}
	}

	return true;
}


bool skelView::LineContainsPoint(int nLineNr, int nPtNr) {

	if(aLines[nLineNr][0] == nPtNr) return true;
	else if(aLines[nLineNr][1] == nPtNr) return true;
	
	return false;
}


int skelView::GetTheOtherPoint (int nLineNr, int nPtNr1) {
	// get the other point of line nLineNr
	// the first point is nPtNr1
	
	if(aLines[nLineNr][0] == nPtNr1) return aLines[nLineNr][1];
	else if(aLines[nLineNr][1] == nPtNr1) return aLines[nLineNr][0];
	
	cout << "Error - get the other point of line " << nLineNr << " - the first point is " << nPtNr1 << " - couldn't find the first point !\n";
	return 0;
}


void skelView::MyPostPick() {
	vtkAssemblyPath* 		pp = NULL;
	vtkAssemblyNode* 		pn = NULL;
	vtkProp*		 		pr = NULL;

	//cout << "something was picked\n";
	
	// Return the previous selected actor aand assembly to their initial state
	Unselect();
	
	pSelectedAssembly	= NULL;
	pSelectedActor		= NULL;

	// the picked assembly
	pSelectedAssembly = pPicker->GetAssembly();
	if(pSelectedAssembly != NULL) {
		pp = pPicker->GetPath();
		if(pp != NULL) {
			pp->InitTraversal();
			while((pn = pp->GetNextNode())) {
				pr = pn->GetProp();
				if(pr != NULL) {
					if(pr->IsA("vtkAssembly")) { 
						pSelectedAssembly = (vtkAssembly*) pr;
					} else {
						if(pr->IsA("vtkActor")) { 
							pSelectedActor = (vtkActor*) pr;
						}
					}
				}
			}			
		}
	}
	
	pp = NULL;
	pn = NULL;
	pr = NULL;

	Select();
	
}

void skelView::Unselect() {
	// Reset the color of the previously selected actor

	if(pSelectedActor != NULL) {
		pSelectedActor->SetProperty(pNormalProperties);
	}
}

void skelView::Select() {
	// Color the selected actor
	if(pSelectedActor != NULL) {
		pSelectedActor->SetProperty(pSelectedProperties);
	}

}

void skelView::MyOnKeyPress(char c) {

	//cout << "a key was pressed: " << c << "\n";	
	
	if((c == 'x') || (c == 'X')) {
		RotateSelectedBoneX(3);
	} 
	else {
		if((c == 'y') || (c == 'Y')) {
			RotateSelectedBoneY(3);
		} 
		else {
			if((c == 'z') || (c == 'Z')) {
				RotateSelectedBoneZ(3);
			} 
			else {
				if((c == 'c') || (c == 'C')) {
					RotateSelectedBoneX(-3);
				} 
				else {
					if((c == 't') || (c == 'T')) {
							RotateSelectedBoneY(-3);
					} 
					else {
						if((c == 'a') || (c == 'A')) {
							RotateSelectedBoneZ(-3);
						} 
					}
				}
			}
		}
	}
}



void skelView::MyUserProc() {
	//cout << "user proc\n";
	GenerateOutputFile();
}


void skelView::WriteOutput() {
	// open the file
	FILE* fh = fopen(cOutputFileName, "w+");
	if(fh == NULL) {
		cout << "Error - Could not open file " << cOutputFileName << " for writing... \n";
		return;
	}
	cout << "File " << cOutputFileName << " successfuly opened for writing.\n";

	WriteOterStuffToFile(fh);
	WriteAllTransformsToFile(fh);

	fclose(fh);
	cout << "Closed output file.\n";
}


void skelView::WriteOterStuffToFile(FILE* fh) {
	FILE* fhi;
	char ln[200];

	// copy the input file up to te TRANFORMATIONS section
	// open the input file
	fhi = fopen(cInputFileName, "r");
	if(fhi == NULL) {
		cout << "Could not open file " << cInputFileName << " for reading... \n";
		return;
	}
	cout << "File " << cInputFileName << " successfuly opened for reading.\n";
	
	strcpy(ln, "");
	while(fgets(ln, 200, fhi) != NULL) {
		// not EOF yet
		// delete the spaces at the begining of the line
		//while(isspace(ln[0]) != 0) {
		//	strcpy(ln, ln+1);
		//}
		//cout << ln;
		
		if(strlen(ln) > 0) {
			if(isalpha(ln[0]) != 0) {
				if(strncmp(ln, "TRANSFORMATIONS", 15) == 0) {
					fputs(ln, fh);
					break;
				}
			}
		}

		fputs(ln, fh);

		strcpy(ln, "");
	}
	fclose(fhi);
}

void skelView::StoreTransformation(vtkMatrix4x4* pm, int index) {
	int istart = index * 4;
	float a, b, c, d;
	
	if(istart >= (nMAXNrOfActors * 4)) {
		cout << "Error - too many transformations ... \n";
		return;
	}

	for(int j=0; j < 4; j++) {
		a = pm->GetElement(0, j);
		b = pm->GetElement(1, j);
		c = pm->GetElement(2, j);
		d = pm->GetElement(3, j);

		if(fabs(a) < 1.0e-09) a = 0;
		if(fabs(b) < 1.0e-09) b = 0;
		if(fabs(c) < 1.0e-09) c = 0;
		if(fabs(d) < 1.0e-09) d = 0;

		aTransform[istart + j][0] = a;
		aTransform[istart + j][1] = b;
		aTransform[istart + j][2] = c;
		aTransform[istart + j][3] = d;
	}
}

void skelView::StoreAllTransforms() {
	vtkAssemblyPath* 		pap = NULL;
	vtkAssemblyNode* 		pan = NULL;
	vtkProp*		 		ppr = NULL;	
	vtkMatrix4x4*			pm	= NULL;
	int ai = 0;

	ptheAssembly->InitPathTraversal();

	while((pap = ptheAssembly->GetNextPath())) {
		pap->InitTraversal();
		
		pan = pap->GetLastNode();
		if(pan != NULL) {
			ppr = pan->GetProp();
			if(ppr != NULL) {
				pm = pan->GetMatrix();
				if(pm != NULL) {
					ai = GetIndexOfActor(ppr);
					if(ai != -1) {
						StoreTransformation(pm, ai);
					}
				}
			}
		}
	}
}

int skelView::GetIndexOfActor(vtkProp* pa) {
	for(int i=0; i < nNrOfActors; i++) {
		if(aActors[i] == pa) return i;
	}
	return -1;
}


void skelView::WriteAllTransformsToFile(FILE* fh) {
	int k = 0;
	char t[20];
	
	for(int i=0; i < (nNrOfActors * 4); i++) {
		strcpy(t, "");
		sprintf(t, "%f %f %f %f \n", aTransform[i][0], aTransform[i][1], aTransform[i][2], aTransform[i][3]);
		fputs(t, fh);
		k++;
		if(k >= 4) {
			k = 0;
			fputs("\n", fh);
		}
	}
}

void skelView::RenderSkeleton() {
	//cout << "Show the skeleton...\n";
	pRenWin->Render();
}

void skelView::mousePressEvent (QMouseEvent * event) {

	if(event->state() & Qt::ShiftButton) {
		//cout <<"Shift";	
		
		int x = event->x();
		int y = this->width() - event->y();

		//cout << "mouse click - pick at: "<< x << " " << y << "\n";
		pPicker->Pick(x, y, 0, pRen);
		MyPostPick();
		pRenWin->Render();
	}
	else {
		RenderWindowInteractor::mousePressEvent(event);
	}
}

void skelView::keyPressEvent(QKeyEvent* e ) {
	//cout << "+"<< e->key() << "+\n";
	int k = e->key();
	if(k == Qt::Key_X) MyOnKeyPress('x');
     	if(k == Qt::Key_Y) MyOnKeyPress('y');
	if(k == Qt::Key_Z) MyOnKeyPress('z');
	if(k == Qt::Key_C) MyOnKeyPress('c');
     	if(k == Qt::Key_A) MyOnKeyPress('a');
	if(k == Qt::Key_T) MyOnKeyPress('t');

	if(k == Qt::Key_U) MyUserProc();
}


void skelView::GenerateOutputFile() {
	StoreAllTransforms();
	WriteOutput();
}



bool skelView::SetInputFileName(char* fn) {
	if(fn == NULL) return false;
	cout << "Input file name set to : " << fn << endl;
	strcpy(cInputFileName, fn);
	cout << "Input file name set to : " << cInputFileName << endl;
	Cleanup();
	InitStructure();
	InitState();
	return true;
}

bool skelView::SetOutputFileName(char* fn) {
	if(fn == NULL) return false;
	strcpy(cOutputFileName, fn);
	return true;
}


bool skelView::RotateSelectedBoneX(int deg) {
	if(pSelectedAssembly != NULL) {
		pSelectedAssembly->RotateWXYZ(deg, 1, 0, 0);
		pRenWin->Render();
		return true;
	} 
	return false;
}

bool skelView::RotateSelectedBoneY(int deg) {
	if(pSelectedAssembly != NULL) {
		pSelectedAssembly->RotateWXYZ(deg, 0, 1, 0);
		pRenWin->Render();
		return true;
	} 
	return false;
}

bool skelView::RotateSelectedBoneZ(int deg) {
	if(pSelectedAssembly != NULL) {
		pSelectedAssembly->RotateWXYZ(deg, 0, 0, 1);
		pRenWin->Render();
		return true;
	}
	return false;
}

void skelView::showEvent ( QShowEvent * ) {
	cout << "\nSkelView show event";
	//RenderSkeleton();
}
