//////////////////////////////////////////////////////////////
// SkeView class definition
//////////////////////////////////////////////////////////////

// VTK includes

#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkConeSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkLineSource.h"
#include "vtkActor.h"
#include "vtkAssemblyNode.h"
#include "vtkAssembly.h"
#include "vtkCellPicker.h"
#include "vtkInteractorStyleUser.h"
#include "vtkMatrix4x4.h"


#include "renderwindowinteractor.h"

#define nMAXNrOfPoints 50
#define nMAXNrOfActors 50

// name of the input and output files
#define cInputFN 		"no_input_file_specified.vtk"
#define cOutputFN 		"output.txt"


class skelView  : public RenderWindowInteractor {
	Q_OBJECT

public:
	skelView(QWidget *parent = 0, const char *name = 0);
	skelView( QGLFormat glf, QWidget *parent = 0, const char *name = 0);

	~skelView();

	
	// RenderSkeleton - You don't really need to call this method, 
	// unless the widget fails to redraw itsels in some particular situation.
	void RenderSkeleton();
	

	bool SetInputFileName(char* fn);
	bool SetOutputFileName(char* fn);

public slots:
	// Rotates the selected bone arround the X, Y or Z axis.
	//	If no bone is selected, nothing happens
	bool RotateSelectedBoneX(int deg);
	bool RotateSelectedBoneY(int deg);
	bool RotateSelectedBoneZ(int deg);
	
	// GenerateOutputFile - writes the output to the file 
	//		defined by cOutputFilename 
	void GenerateOutputFile();

protected:
	virtual void mousePressEvent (QMouseEvent* event);
	virtual void keyPressEvent(QKeyEvent* e );
	virtual void showEvent ( QShowEvent * );

private:
	vtkRenderer* 				pRen;	
	vtkRenderWindow* 			pRenWin;
	vtkCellPicker*				pPicker;
	
	// Selected and Unselected properties
	vtkProperty* pSelectedProperties;
	vtkProperty* pNormalProperties;
		
	int nNrOfPoints;
	int nNrOfActors;
	
	// points array
	float				aPoints[nMAXNrOfPoints][3];
	// lines array
	int				aLines[nMAXNrOfActors][3];
	// Traaansformations array
	float				aTransform[nMAXNrOfActors*4][4];
	// Line Source array
	vtkLineSource*		aLineSources[nMAXNrOfActors];
	// mappers array
	vtkPolyDataMapper*	aMappers[nMAXNrOfActors];		
	// actors array
	vtkActor*			aActors[nMAXNrOfActors];
	// assembly array
	vtkAssembly* 		aAssemblies[nMAXNrOfActors];
	// the assembly
	vtkAssembly* ptheAssembly;
	
	vtkAssembly*	pSelectedAssembly;
	vtkActor*		pSelectedActor;

private:
	bool InitStructure();
	bool InitState();
	void Cleanup();

	void SetNormalProperties();
	void SetSelectedProperties();

	bool ReadPointsLinesAndTransformations();
	bool MakeEachLineAnActor();
	bool BuildActorHierarchy();
	bool AddActors(vtkAssembly* prootAssembly, int nPointNr, bool aLineFlags[nMAXNrOfActors]);
	bool LineContainsPoint(int nLineNr, int nPtNr);
	int GetTheOtherPoint (int nLineNr, int nPtNr1);

	void MyPostPick();
	void Unselect();
	void Select();

	void MyOnKeyPress(char c);
	void MyUserProc();

	void WriteOutput();
	void WriteOterStuffToFile(FILE* fh);
	void StoreTransformation(vtkMatrix4x4* pm, int index);
	void StoreAllTransforms();
	int GetIndexOfActor(vtkProp* pa);
	void WriteAllTransformsToFile(FILE* fh);

	char cInputFileName[100];
	char cOutputFileName[100];
};

