#include <qtimer.h>
#include "QtInterface.h"

#if defined(Q_CC_MSVC)
#pragma warning(disable:4305) //init: truncation from const double to float
#endif



skeletonFile::skeletonFile(QWidget* parent, const char* name)
  : QGLWidget(parent, name)
{
  
}

skeletonFile::~skeletonFile()
{ 
  makeCurrent();  
}

void skeletonFile::cleanSkel(void)
{
  if(vpt.size() != 0)
    vpt.clear();

  if(vcL.size() != 0)
    vcL.clear();

  if(vMaxDis.size() != 0)
    vMaxDis.clear();
  
  if(vecmat.size() != 0)
    vecmat.clear();

  if(vGeom.size() != 0)
    vGeom.clear();
}

void skeletonFile::setXRotation(int degrees)
{
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glRotatef(degrees, 1, 0, 0);
  updateGL();
}


void skeletonFile::setYRotation(int degrees)
{
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();  
  glRotatef(degrees, 0, 1, 0);
  updateGL();
}


void skeletonFile::setZRotation(int degrees)
{
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity(); 
  glRotatef(degrees, 0, 0, 1);
  updateGL();
}







void skeletonFile::SetSkeleton(void)
     
{
  char str[256], buf[100];
  float x1, y1, z1, x2, y2, z2;
  int numpts, p1idx, p2idx;
  int count;
  SbMatrix mat;
  FILE* tskelFile;
  //tskelFile = fopen("newTskel/skel020.vtk", "r");
  tskelFile = fopen("output21.txt", "r");
  
  fscanf(tskelFile, "%[^\n]", str);
  while(!feof(tskelFile))
    {
      int filepos;
      if(strncmp(str, "POINTS", 6) == 0)
	{
	  if(sscanf(str, "%s%d", &buf, &count) != 2)
	    {
	      cerr<<"Error in new Tskel File...exiting"<<endl;
	      exit(1);
	    }       
	  for(int i = 0; i < count; i++)
	    {
	      fscanf(tskelFile, "%f%f%f", &x1, &y1, &z1);
	      vpt.push_back(point(x1, y1, z1));		 
	    }	            
	}
      
      else if(strncmp(str, "LINES", 5) == 0)
	{
	  
	  if(sscanf(str, "%s%d", &buf, &count) != 2)
	    {
	      cerr<<"Error in new Tskel File...exiting"<<endl;
	      exit(1);
	    }
	  
	  for(int i = 0; i < count; i++)
	    {
	      fscanf(tskelFile, "%d%d%d", &numpts, &p1idx, &p2idx);
	      vcL.push_back(connectLines(numpts, p1idx, p2idx));
	    }
	  
	}
      
      
	  else if(strncmp(str, "MAXDIS", 6) == 0)
	    {
	      if(sscanf(str, "%s%d", &buf, &count) != 2)
		{
		  cerr<<"Error in new Tskel File...exiting"<<endl;
		  exit(1);
		}
	      
	      for(int i = 0; i < count; i++)
		{
		  fscanf(tskelFile, "%f", &x1);
		  vMaxDis.push_back(x1/1.0);
		}
	      
	    }
      
	  else if(strncmp(str, "TRANSFORMATIONS", 15) == 0)
	    {
	      if(sscanf(str, "%s%d", &buf, &count) != 2)
		{
		  cerr<<"Error in new Tskel File...exiting"<<endl;
		  exit(1);
		}

	      for(int i = 0; i < count; i++)
		{
		  
		  fscanf(tskelFile, "%f%f%f%f", &mat[0][0], &mat[0][1], &mat[0][2], &mat[0][3]);
		  fscanf(tskelFile, "%f%f%f%f", &mat[1][0], &mat[1][1], &mat[1][2], &mat[1][3]);
		  fscanf(tskelFile, "%f%f%f%f", &mat[2][0], &mat[2][1], &mat[2][2], &mat[2][3]);
		  fscanf(tskelFile, "%f%f%f%f", &mat[3][0], &mat[3][1], &mat[3][2], &mat[3][3]);
		  vecmat.push_back(mat);
		}
	      
	    }

      
      fscanf(tskelFile, "\n");
      fscanf(tskelFile, "%[^\n]", str);
    }
  for (int i = 0; i < vpt.size(); i++)
    {
      jointGeometry geom_ini;
	  vGeom.push_back(geom_ini);
    }

  defineJointGeometry();
  
}



void skeletonFile::renderCuboids()
{
    
 
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
  
  for(int numbones = 0; numbones < (int)vcL.size(); numbones++)
    {
      SbVec3f bonePoint1(vpt[vcL[numbones].getp1Index()].getX(), vpt[vcL[numbones].getp1Index()].getY(), vpt[vcL[numbones].getp1Index()].getZ());
      SbVec3f bonePoint2(vpt[vcL[numbones].getp2Index()].getX(), vpt[vcL[numbones].getp2Index()].getY(), vpt[vcL[numbones].getp2Index()].getZ());
      SbVec3f bonePointTr1, bonePointTr2;
      
      vecmat[numbones].multVecMatrix(bonePoint1, bonePointTr1);
      vecmat[numbones].multVecMatrix(bonePoint2, bonePointTr2);
      float pt1X, pt1Y, pt1Z, pt2X, pt2Y, pt2Z;
      
      pt1X = bonePointTr1[0];
      pt1Y = bonePointTr1[1];
      pt1Z = bonePointTr1[2];
      pt2X = bonePointTr2[0];
      pt2Y = bonePointTr2[1];
      pt2Z = bonePointTr2[2];	 	
      
      /*************Sat October 5****************/
      line sline(point(pt1X, pt1Z, pt1Y), point(pt2X, pt2Z, pt2Y));
      point newp1, newp2;
      newp1 = sline.getPointOnLineFromP1(0.15);
      newp2 = sline.getPointOnLineFromP2(0.15);
      
      /***may 08, 2003****/
      SbVec3f vport1(newp1.getX()-27-128, newp1.getY()-458-256, newp1.getZ()+33-128);
      SbVec3f vport2(newp2.getX()-27-128, newp2.getY()-458-256, newp2.getZ()+33-128);
      SbMatrix view(modelview[0], modelview[1],modelview[2],modelview[3],modelview[4],modelview[5],modelview[6],modelview[7],modelview[8],modelview[9],modelview[10],modelview[11],modelview[12],modelview[13],modelview[14],modelview[15]);
      
      SbMatrix invView, transpose, invTranspose;
      invView = view.inverse();
      transpose = view.transpose();
      invTranspose = transpose.inverse();
      SbVec3f a1, a2;
      view.multVecMatrix(vport1, a1);
      view.multVecMatrix(vport2, a2);      
      point po1(a1[0]+128, a1[1]+256, a1[2]+128);
      point po2(a2[0]+128, a2[1]+256, a2[2]+128);
      
      cout<<"vport1 "<<vport1[0]<<" "<<vport1[1]<<" "<<vport1[2]<<endl;
      cubicalBox cboxTr(po1, po2, vMaxDis[numbones]/1.0);
      /*******may 08, 2003 ends****/
      
      //cubicalBox cboxTr(newp1-point(27, 458, -33), newp2-point(27, 458, -33), vMaxDis[numbones]/1.0);
      
      /*************Sat October 5...End*******************/
      //cubicalBox cboxTr(point((pt1X-27), (pt1Z-458), (pt1Y+33)), point((pt2X-27), (pt2Z-458), (pt2Y+33)), vMaxDis[numbones]);
      cboxTr.set12Lines();	    
      
      SbMatrix inv;
      inv = vecmat[numbones].inverse();
      
      
      for(float z = -100.0; z < 500.0 ; z = z + 0.5)
	{
	  if(cboxTr.isIntersectingZPlane(z))
	    {  
	      vector<point> vlist;
	      vector<point> sortpts;
	      vlist = cboxTr.getIntersectingVerticesWithZPlane(z);
	      sortpts = sortPoints(vlist);
	      //vlist = cbox.getIntersectingVerticesWithZPlane(z);
	      
	      glBegin(GL_POLYGON);
	      for(int i = 0; i<sortpts.size(); i++)
		{
		  float X = sortpts[i].getX();
		  float Y = sortpts[i].getY();
		  float Z = sortpts[i].getZ();
		  
		  SbVec3f v1(X-128, Y-256, Z-128);
		  SbVec3f v, t;
		  view.multVecMatrix(v1,v);
		  invView.multVecMatrix(v1, t);
		  
		  SbVec3f Vtr((t[0]+128+27), (t[2]+128-33), (t[1]+256+458));
		  SbVec3f Tex;
		  inv.multVecMatrix(Vtr, Tex);
		  
		  float Xtex = (Tex[0]-27);
		  float Ytex = (Tex[2]-458);
		  float Ztex = (Tex[1]+33);
		  //glColor4f(0.5, 0.5, 0.5, 0.0);
		  //glVertex3f(170+X, Y, Z); 	   
		  //glTexCoord3f(X/256, Y/512, Z/128); glVertex3f(160+X, 512-Y, Z); 
		      //glTexCoord3f(Xtex/256, Ytex/512, Ztex/256); glVertex3f(-X, -Y, Z); 
		  
		      /****New test*********/
		  
		  
		      
		  
		  glTexCoord3f(Xtex/256, Ytex/512, Ztex/256); glVertex3f(-v[0], -v[1], v[2]); 
		  
		  
		  
		  // glTexCoord3f(Xtex/256, Ytex/512, Ztex/256); glVertex3f(-X+128.0, -Y+256.0, Z-128.0); 
		  
		  
		      //Original pose
		      //glTexCoord3f(Xtex/256, Ytex/512, Ztex/256); glVertex3f(X+100, -Y+420, -Z); 
		  
		  
		}
	      glEnd();
	      glFlush();
		  
	    }
	}
      
    }
  
}


void skeletonFile::renderJointSpheres(bool pbuffer, float factor)
{
  float radSphere[vpt.size()];
  SbMatrix sbmat[vpt.size()];
 
  for (int i = 0; i < vpt.size(); i++)
    radSphere[i] = 0;

  for(int i = 0; i < vcL.size(); i++)
    {
      if(radSphere[vcL[i].getp1Index()] < vMaxDis[i])
	radSphere[vcL[i].getp1Index()] = vMaxDis[i];
      if(radSphere[vcL[i].getp2Index()] < vMaxDis[i])
	radSphere[vcL[i].getp2Index()] = vMaxDis[i];	
      
      sbmat[vcL[i].getp1Index()] = vecmat[i];
      sbmat[vcL[i].getp2Index()] = vecmat[i];

    }

  SbMatrix temp;
  temp = sbmat[21];
  // cout<<temp[0][0]<<" "<<temp[0][1]<<" "<<temp[0][2]<<" "<<temp[0][3]<<endl;

  for(int numSpheres = 0; numSpheres < vpt.size(); numSpheres++)
    {
      SbVec3f old(vpt[numSpheres].getX(), vpt[numSpheres].getY(), vpt[numSpheres].getZ());
      SbVec3f newer;
      sbmat[numSpheres].multVecMatrix(old, newer);

      SbMatrix inv;
      float radius;
      float zPlane;
      float zMax, zMin;

      /*************** MODIFIED*******************/
      //radius = radSphere[numSpheres]/2.0;
      radius = 26.0;
      /******************************************/

      zMax = radius;
      zMin = (-radius);
      
      inv = sbmat[numSpheres].inverse();
     
      for(float zPlane = zMin; zPlane <= zMax; zPlane = zPlane + 0.5)
	{

	  float yRadius;
	  

	  //Assume x=0 to find the range of y-values. Assume x<y<z
	  //Since x=0, DT = y*4 + (z-y)*3 = 3*z + y
	 /*  yRadius = radius - abs((int)zPlane)*3; */
/* 	  if(yRadius > abs((int)zPlane)) */
/* 	    { */
/* 	      //Clearly against the assumption - take case x<z<y */
/* 	      //equation becomes DT = z*4 + (y-z)*3 = z + 3y */
/* 	      yRadius = (radius - abs((int)zPlane))/3; */
/* 	      if((yRadius<0) || (yRadius<abs((int)zPlane))) */
/* 		yRadius = -1; */
/* 	    } */
	    yRadius = sqrt(radius*radius - zPlane*zPlane);

	    //yRadius = radius;
	    //yRadius = 26;
      
	  glBegin(GL_POLYGON);
	  for(float theta = 0.0; theta < 2.0*PI; theta = theta + 0.1)
	    {
	  /*     float xx = vpt[numSpheres].getX() + yRadius*cos(theta); */
/* 	      float yy = vpt[numSpheres].getY() + yRadius*sin(theta); */
/* 	      float zz = vpt[numSpheres].getZ() + zPlane; */
	      float Vxx = newer[0] - 27 + yRadius*cos(theta)*factor-0.30;
	      float Vyy = newer[2] - 458 + yRadius*sin(theta)*factor;
	      float Vzz = newer[1] + 33 + zPlane;

	      float xx = newer[0] - 27 + yRadius*cos(theta)*1.0;
	      float yy = newer[2] - 458 + yRadius*sin(theta)*1.0;
	      float zz = newer[1] + 33 + zPlane;
	  
	      SbVec3f v(xx+27, zz-33, yy+458);
	      SbVec3f v1;
	      inv.multVecMatrix(v, v1);	     
	      
	      if(pbuffer)
	      glTexCoord3f((v1[0]-27)/256, (v1[2]-458)/512, (v1[1]+33)/256);
	      glVertex3f(Vxx+270, Vyy+460, -Vzz); 
	      
	    }
	  glEnd();
	  glFlush();
	  
	}


    }
  
}


void skeletonFile::defineJointGeometry(void)
{
  for(int numline = 0; numline < vcL.size(); numline++)
    {  
      vGeom[vcL[numline].getp1Index()].setJointGeometry(vcL[numline].getp1Index(), numline);
      vGeom[vcL[numline].getp2Index()].setJointGeometry(vcL[numline].getp2Index(), numline);      
    
    }
  cout<<"vcL size "<<vcL.size()<<endl;
  for(int joint = 0; joint < vGeom.size(); joint++)  
    {
    
      if(vGeom[joint].getNumberOfConnectedBones() == 2)
	{
       
	  vector<int> connectedBones;
	  connectedBones = vGeom[joint].getConnectedBoneIdx();      
	  SbVec3f jointOri(vpt[joint].getX(), vpt[joint].getY(), vpt[joint].getZ());
	  SbVec3f jointTr;
	  vecmat[connectedBones[0]].multVecMatrix(jointOri, jointTr);
	  point jpt(jointTr[0], jointTr[1], jointTr[2]);

	  for(int j = 0; j < connectedBones.size(); j++)
	    {
	      point p;
	      if(vcL[connectedBones[j]].getp1Index() != joint)	  
		{
		  SbVec3f p1Ori(vpt[vcL[connectedBones[j]].getp1Index()].getX(),  vpt[vcL[connectedBones[j]].getp1Index()].getY(),  vpt[vcL[connectedBones[j]].getp1Index()].getZ());
		  SbVec3f p1Tr;
		  vecmat[connectedBones[j]].multVecMatrix(p1Ori, p1Tr);
		  p = point(p1Tr[0], p1Tr[1], p1Tr[2]) - jpt;
		}
	      else	   
		{
		  SbVec3f p1Ori(vpt[vcL[connectedBones[j]].getp2Index()].getX(),  vpt[vcL[connectedBones[j]].getp2Index()].getY(),  vpt[vcL[connectedBones[j]].getp2Index()].getZ());
		  SbVec3f p1Tr;
		  vecmat[connectedBones[j]].multVecMatrix(p1Ori, p1Tr);
		  p = point(p1Tr[0], p1Tr[1], p1Tr[2]) - jpt;
		}
	      
	      SbVec3f vnorm(p.getX(), p.getY(), p.getZ());
	      float vMag = vnorm.normalize();
	      //cout<<"vnorm[0] "<<vnorm[0]<<" vnorm[1] "<<vnorm[1]<<" vnorm[2] "<<vnorm[2]<<endl;
	      switch(j)
		{
		case 0:
		  vGeom[joint].setV1(vnorm);
		  vGeom[joint].setv1Mag(vMag);
		  break;
		case 1:
		  vGeom[joint].setV2(vnorm);
		  vGeom[joint].setv2Mag(vMag);
		  break;
		default:
		  break;
		}		
	      
	    }

	  SbVec3f vp;
	  SbVec3f up;
	  vp = (vGeom[joint].getV1()).cross(vGeom[joint].getV2());
	  up = vGeom[joint].getV1() + vGeom[joint].getV2();
	  float vpMag = vp.normalize();
	  float upMag = up.normalize();
	  float maxR, minR;
	  if(vMaxDis[connectedBones[0]] >= vMaxDis[connectedBones[1]])
	    {
	      maxR = SQRT2*vMaxDis[connectedBones[0]];
	      minR = SQRT2*vMaxDis[connectedBones[1]];
	    }
	  else
	    {
	      maxR = SQRT2*vMaxDis[connectedBones[1]];
	      minR = SQRT2*vMaxDis[connectedBones[0]];
	    }
	 
	  for(float theta = 0; theta<2*PI; theta = theta + PI/2)
	    {
	      float x = jpt.getX() + maxR*cos(theta)*vp[0] + maxR*sin(theta)*up[0];
	      float y = jpt.getY() + maxR*cos(theta)*vp[1] + maxR*sin(theta)*up[1];
	      float z = jpt.getZ() + maxR*cos(theta)*vp[2] + maxR*sin(theta)*up[2]; 
	      vGeom[joint].setMidPlane(point(x, y, z));	      
	    }
	  
	  SbVec3f u1, u2;	  
	  SbVec3f v2neg;
	  v2neg = vGeom[joint].getV2();
	  v2neg.negate();
	  u1 = vp.cross(vGeom[joint].getV1());
	  u2 = vp.cross(v2neg);

	  for(int j = 0; j < connectedBones.size(); j++)
	    {
	      
	      line l;
	      if(vcL[connectedBones[j]].getp1Index() == joint)		
		l.setP1P2(vpt[joint], vpt[vcL[connectedBones[j]].getp2Index()]);
	      else
		l.setP1P2(vpt[joint], vpt[vcL[connectedBones[j]].getp1Index()]);
	      
	      point pt;
	      pt = l.getPointOnLineFromP1(0.15);
	      SbVec3f ptOri(pt.getX(), pt.getY(), pt.getZ());
	      SbVec3f ptTr;
	      vecmat[connectedBones[j]].multVecMatrix(ptOri, ptTr);
	      float rad;
	      rad = SQRT2*vMaxDis[connectedBones[j]];
	      
	      switch(j)
		{
		case 0:		  
		  for(float theta = 0; theta<2*PI; theta = theta + PI/2)
		    {
		      float x = ptTr[0] + rad*cos(theta)*vp[0] + rad*sin(theta)*u1[0];
		      float y = ptTr[1] + rad*cos(theta)*vp[1] + rad*sin(theta)*u1[1];
		      float z = ptTr[2] + rad*cos(theta)*vp[2] + rad*sin(theta)*u1[2]; 
		      vGeom[joint].setPlane1(point(x, y, z));	      
		    }
		  break;
		  
		case 1:
		  for(float theta = 0; theta<2*PI; theta = theta + PI/2)
		    {
		      float x = ptTr[0] + rad*cos(theta)*vp[0] + rad*sin(theta)*u2[0];
		      float y = ptTr[1] + rad*cos(theta)*vp[1] + rad*sin(theta)*u2[1];
		      float z = ptTr[2] + rad*cos(theta)*vp[2] + rad*sin(theta)*u2[2]; 
		      vGeom[joint].setPlane2(point(x, y, z));	      
		    }
		  break;
		  
		}
	      
	      

	      
	    }

	  
	}
      
    }

  
 
  glDisable(GL_TEXTURE_3D);
  for(int i = 0; i < vGeom.size(); i++)
    {
      
      if(vGeom[i].getNumberOfConnectedBones() == 2)
        {	 
	  
	  vector<int> cBones;
	  cBones = vGeom[i].getConnectedBoneIdx();
	  
	  SbVec3f v1tmp, v2tmp;
	  // v1tmp = vGeom[i].getV1();
	  //v2tmp = vGeom[i].getV2();
	  v1tmp = (vGeom[i].getv1Mag())*(vGeom[i].getV1());
	  v2tmp = (vGeom[i].getv2Mag())*(vGeom[i].getV2());

	  point pp, pp1, pp2;
	  //pp = vpt[i];
	  SbVec3f ppOri(vpt[i].getX(), vpt[i].getY(), vpt[i].getZ());
	  SbVec3f ppTr;
	  vecmat[cBones[0]].multVecMatrix(ppOri, ppTr);
	  pp.setXYZ(ppTr[0], ppTr[1], ppTr[2]);
	
	  pp1 = pp + point(v1tmp[0], v1tmp[1], v1tmp[2]);
	  pp2 = pp + point(v2tmp[0], v2tmp[1], v2tmp[2]);
	  glLineWidth(3);
	  glColor4f(1.0, 1.0, 0.0, 1.0);      
	  glBegin(GL_LINES);
	  glVertex3f(pp.getX()-27+100, -(pp.getZ()-458-420), -pp.getY()-33);
	  glVertex3f(pp1.getX()-27+100, -(pp1.getZ()-458-420), -pp1.getY()-33);
	  glVertex3f(pp.getX()-27+100, -(pp.getZ()-458-420), -pp.getY()-33);
	  glVertex3f(pp2.getX()-27+100, -(pp2.getZ()-458-420), -pp2.getY()-33);
	  glEnd();
	  glFlush();	  
	  glColor4f(0.2, 0.2, 0.3, 0.0055);

	  vector<point> md, plane1, plane2;
	  md = vGeom[i].getMidPlane();
	  plane1 = vGeom[i].getPlane1();
	  plane2 = vGeom[i].getPlane2();
	 	    
/* 	  glPointSize(3); */
	  
/* 	  for(int i = 0; i < md.size(); i++) */
/* 	    { */
/* 	      glBegin(GL_LINE_STRIP); */
/* 	      glVertex3f(plane1[i].getX()-27+270, plane1[i].getZ()-458+460, -plane1[i].getY()-33); */
/* 	      glVertex3f(md[i].getX()-27+270, md[i].getZ()-458+460, -md[i].getY()-33);	      */
/* 	      glVertex3f(plane2[i].getX()-27+270, plane2[i].getZ()-458+460, -plane2[i].getY()-33); */
/* 	      glEnd(); */
/* 	    }	 */
/* 	  glFlush();	  */  
	  

	}	  
    }
  glEnable(GL_TEXTURE_3D);
}


void skeletonFile::renderJointGeometry(void)
{
  /****TESTING AREA**********/
  cout<<"ModelView matrix"<<endl;
  for(int i=0; i<4; i++)
    {
      for(int j=0; j<4; j++)
	cout<<modelview[4*i+j]<<" ";
      cout<<endl;
    }
  SbMatrix view(modelview[0], modelview[1],modelview[2],modelview[3],modelview[4],modelview[5],modelview[6],modelview[7],modelview[8],modelview[9],modelview[10],modelview[11],modelview[12],modelview[13],modelview[14],modelview[15]);
  SbMatrix invView;
  invView = view.inverse();
  /**************************/
  
  glColor4f(0.2, 0.2, 0.3, 0.0055);
  for(int joint = 0; joint < vGeom.size(); joint++)
    {
      if(vGeom[joint].getNumberOfConnectedBones() == 2)
	{
	  vector<point> midPlane;
	  vector<point> Plane1;
	  vector<point> Plane2;
	  midPlane = vGeom[joint].getMidPlane();
	  Plane1 = vGeom[joint].getPlane1();
	  Plane2 = vGeom[joint].getPlane2();
	  vector<line> vline1;
	  vector<line> vline2;
	  vector<int> connectedBones;
       

	  connectedBones = vGeom[joint].getConnectedBoneIdx();
	  
	  for(int i = 0; i < Plane1.size(); i++)
	    vline1.push_back(line(Plane1[i], Plane1[(i+1)%Plane1.size()]));
	  for(int i = 0; i < Plane2.size(); i++)
	    vline2.push_back(line(Plane2[i], Plane2[(i+1)%Plane2.size()]));
	  for(int i = 0; i < midPlane.size(); i++)
	    {
	      vline1.push_back(line(Plane1[i], midPlane[i]));
	      vline2.push_back(line(midPlane[i], Plane2[i]));	  
	    }
	  
	  SbMatrix inv1, inv2;
	  inv1 = vecmat[connectedBones[0]].inverse();
	  inv2 = vecmat[connectedBones[1]].inverse();
	  
	  for(float zPlane = -100.0; zPlane < 500.0; zPlane += 0.5)
	    {
	      vector<Vertex> vlist;
	      
	      for(int line1 = 0; line1 < vline1.size(); line1++)
		{
		  point p1, p2;
		  p1 = vline1[line1].getP1();
		  p2 = vline1[line1].getP2();
		  //line l(point(p1.getX()-27, p1.getZ()-458, p1.getY()+33), point(p2.getX()-27, p2.getZ()-458, p2.getY()+33));

		  /***************may 8, 2003****************/
		  SbVec3f vport1(p1.getX()-27-128, p1.getZ()-458-256, p1.getY()+33-128);
		  SbVec3f vport2(p2.getX()-27-128, p2.getZ()-458-256, p2.getY()+33-128);
		  
		  SbVec3f a1, a2;
		  view.multVecMatrix(vport1, a1);
		  view.multVecMatrix(vport2, a2);      
		  point po1(a1[0]+128, a1[1]+256, a1[2]+128);
		  point po2(a2[0]+128, a2[1]+256, a2[2]+128);
		  line l(po1, po2);
		  
		  /************may 08, 2003 ends*************/
		  
		  if(l.intersectZPlane(zPlane))		   
		    {
		      point pp;
		      pp = l.intersectPoint(zPlane);


		      SbVec3f v1(pp.getX()-128, pp.getY()-256, pp.getZ()-128);
		      SbVec3f v, t;
		      view.multVecMatrix(v1,v);
		      invView.multVecMatrix(v1, t);
		      point vcoords(v[0],v[1], v[2]);
		      
		      SbVec3f Vtr((t[0]+128+27), (t[2]+128-33), (t[1]+256+458) );
		      //SbVec3f Vtr(pp.getX()+27, pp.getZ()-33, pp.getY()+458);
		      SbVec3f Tex;
		      inv1.multVecMatrix(Vtr, Tex);
		      vlist.push_back(Vertex(vcoords, point((Tex[0]-27)/256, (Tex[2]-458)/512, (Tex[1]+33)/256))); 
		      
		    }
		}
	      
	      for(int line2 = 0; line2 < vline1.size(); line2++)
		 {
		  point p1, p2;
		  p1 = vline2[line2].getP1();
		  p2 = vline2[line2].getP2();
		  //line l(point(p1.getX()-27, p1.getZ()-458, p1.getY()+33), point(p2.getX()-27, p2.getZ()-458, p2.getY()+33));
		  
		  SbVec3f vport1(p1.getX()-27-128, p1.getZ()-458-256, p1.getY()+33-128);
		  SbVec3f vport2(p2.getX()-27-128, p2.getZ()-458-256, p2.getY()+33-128);		  		 		 
		  
		  SbVec3f a1, a2;
		  view.multVecMatrix(vport1, a1);
		  view.multVecMatrix(vport2, a2);      
		  point po1(a1[0]+128, a1[1]+256, a1[2]+128);
		  point po2(a2[0]+128, a2[1]+256, a2[2]+128);
		  line l(po1, po2);
		  

		  if(l.intersectZPlane(zPlane))		   
		    {
		      point pp;
		      pp = l.intersectPoint(zPlane);

		      
		      SbVec3f v1(pp.getX()-128, pp.getY()-256, pp.getZ()-128);
		      SbVec3f v, t;
		      view.multVecMatrix(v1,v);
		      invView.multVecMatrix(v1, t);
		      point vcoords(v[0],v[1], v[2]);
		      
		      SbVec3f Vtr((t[0]+128+27), (t[2]+128-33), (t[1]+256+458) );

		      //SbVec3f Vtr(pp.getX()+27, pp.getZ()-33, pp.getY()+458);
		      SbVec3f Tex;
		      inv2.multVecMatrix(Vtr, Tex);
		      vlist.push_back(Vertex(vcoords, point((Tex[0]-27)/256, (Tex[2]-458)/512, (Tex[1]+33)/256))); 
		      
		    }
		}
	       
	      vector<int> sortedVert;
	    
	      //glDisable(GL_TEXTURE_3D);
	      if(vlist.size() != 0)
		{
		  sortedVert = sortVertices(vlist);
		  glEnable(GL_TEXTURE_3D);
	       
		  
		  
		  glBegin(GL_POLYGON);
		  for(int i = 0; i < vlist.size(); i++)
		    {
		      point coord;
		      point tex;
		      coord = vlist[sortedVert[i]].getPosCoords();
		      tex = vlist[sortedVert[i]].getTexCoords();
		      //cout<<"texcoord "<<tex<<endl;
		      //glTexCoord3f(tex.getX(), tex.getY(), tex.getZ()); glVertex3f(coord.getX()+270, coord.getY()+460, -coord.getZ());
		      glTexCoord3f(tex.getX(), tex.getY(), tex.getZ()); glVertex3f(-coord.getX(), -coord.getY(), coord.getZ());

		      // glTexCoord3f(tex.getX(), tex.getY(), tex.getZ()); glVertex3f(-coord.getX()+128.0, -coord.getY()+256.0, coord.getZ()-128.0);
		      //Original pose
		    

		      //glTexCoord3f(tex.getX(), tex.getY(), tex.getZ()); glVertex3f(100+coord.getX(), -coord.getY()+420, -coord.getZ()); 
		      
		    }
		  glEnd();
		  glFlush();
		  
		}
	       
	       
	       

	      
	    }

     	  /******************Draw Yellow Bones*******************/
	  if(glIsEnabled(GL_TEXTURE_3D))
	    glDisable(GL_TEXTURE_3D);

	  vector<int> cBones;
	  cBones = vGeom[joint].getConnectedBoneIdx();
	  
	  SbVec3f v1tmp, v2tmp;
	  // v1tmp = vGeom[joint].getV1();
	  //v2tmp = vGeom[joint].getV2();
	  v1tmp = (vGeom[joint].getv1Mag())*(vGeom[joint].getV1());
	  v2tmp = (vGeom[joint].getv2Mag())*(vGeom[joint].getV2());

	  point pp, pp1, pp2;
	  //pp = vpt[i];
	  SbVec3f ppOri(vpt[joint].getX(), vpt[joint].getY(), vpt[joint].getZ());
	  SbVec3f ppTr;
	  vecmat[cBones[0]].multVecMatrix(ppOri, ppTr);
	  pp.setXYZ(ppTr[0], ppTr[1], ppTr[2]);
	
	  pp1 = pp + point(v1tmp[0], v1tmp[1], v1tmp[2]);
	  pp2 = pp + point(v2tmp[0], v2tmp[1], v2tmp[2]);

	  /*
	  glLineWidth(3);
	  glColor4f(1.0, 1.0, 0.0, 1.0);      
	  glBegin(GL_LINES);
	  glVertex3f(-(pp.getX()-27)+128.0, -(pp.getZ()-458-256.0), pp.getY()-33-128.0);
	  glVertex3f(-(pp1.getX()-27)+128.0, -(pp1.getZ()-458-256.0), pp1.getY()-33-128.0);
	  glVertex3f(-(pp.getX()-27)+128.0, -(pp.getZ()-458-256.0), pp.getY()-33-128.0);
	  glVertex3f(-(pp2.getX()-27)+128.0, -(pp2.getZ()-458-256.0), pp2.getY()-33-128.0);
	  glEnd();
	  glFlush();	  
	  glColor4f(0.2, 0.2, 0.3, 0.0055);
	  */
	  
	  glEnable(GL_TEXTURE_3D);
	  

	  /********************************************/
	
	  //glDisable(GL_TEXTURE_3D);
	  /* Mapping texture to new geometry (midplane geometry) */
/* 	  for(float zPlane = -100.0; zPlane < 500.0; zPlane += 0.5) */
/* 	    { */
/* 	      vector<point> sectpoly; */
	      
/* 	      for(int j = 0; j < vline1.size(); j++) */
/* 		{ */
		  
/* 		  point p1, p2; */
/* 		  p1 = vline1[j].getP1(); */
/* 		  p2 = vline1[j].getP2(); */
/* 		  line l(point(p1.getX()-27, p1.getZ()-458, p1.getY()+33), point(p2.getX()-27, p2.getZ()-458, p2.getY()+33)); */
		  
/* 		  if(l.intersectZPlane(zPlane))		    */
/* 		    sectpoly.push_back(l.intersectPoint(zPlane)); */
/* 		} */
	      
/* 	      if(sectpoly.size() < 3) */
/* 		{ */
/* 		  vector<line> midLines; */
/* 		  for(int i = 0; i < midPlane.size(); i++) */
/* 		    midLines.push_back(line(midPlane[i], midPlane[(i+1)%midPlane.size()]));		   */
		  
/* 		  for(int i = 0; i < midLines.size(); i++) */
/* 		    { */
/* 		      point p3, p4; */
/* 		      p3 = midLines[i].getP1(); */
/* 		      p4 = midLines[i].getP2(); */
/* 		      line ll(point(p3.getX()-27, p3.getZ()-458, p3.getY()+33), point(p4.getX()-27, p4.getZ()-458, p4.getY()+33)); */
/* 		      if(ll.intersectZPlane(zPlane))			 */
/* 			sectpoly.push_back(ll.intersectPoint(zPlane)); */
		      
/* 		    } */
		  
/* 		} */
	      
/* 	      if(sectpoly.size() >= 3) */
/* 		{		   */
/* 		  sectpoly = sortPoints(sectpoly);		 */
/* 		  glBegin(GL_POLYGON); */
/* 		  for(int i = 0; i < sectpoly.size(); i++) */
/* 		    { */
/* 		      float x = sectpoly[i].getX(); */
/* 		      float y = sectpoly[i].getY(); */
/* 		      float z = sectpoly[i].getZ(); */
		      
/* 		      glVertex3f(x+270, y+460, -z); */
		      
		      
/* 		    } */
/* 		  glEnd(); */
/* 		  glFlush(); */
		  
/* 		} */
	      
/* 	    } */
	  /*Mapping texture - Ends */


	  /* Draw Lines - Start */
/* 	  glDisable(GL_TEXTURE_3D); */
/* 	  for(int i = 0; i < vline1.size(); i++) */
/* 	    { */
/* 	      point p1tmp, p2tmp; */
/* 	      p1tmp = vline1[i].getP1(); */
/* 	      p2tmp = vline1[i].getP2(); */
/* 	      glBegin(GL_LINES); */
/* 	      glVertex3f(p1tmp.getX()-27+270, p1tmp.getZ()-458+460, -p1tmp.getY()-33); */
/* 	      glVertex3f(p2tmp.getX()-27+270, p2tmp.getZ()-458+460, -p2tmp.getY()-33); */
/* 	      glEnd();	       */
/* 	      glFlush(); */
/* 	    } */
/* 	  glEnable(GL_TEXTURE_3D); */
	  
	  /* Draw Lines - End */

 
	}
      
    }
 
}

